/* AR model coefficients calculation using Burg algorithm 
   by P.P
   changed by JK to avoid division by zero 
   changed output format (skipped first coefficient and changed sign) */

#include <math.h>
#include <string.h>

#ifndef MAKING_MEX
#include <stdlib.h>
#include <stdio.h>
#include "kalmse.h"

#define SAFEMALLOC(ptr,a,b) ptr=safemalloc((a)*(b)*sizeof(double)) ;
#define SAFEMALLOCINT(ptr,a,b) ptr=safemalloc((a)*(b)*sizeof(int)) ;
#define mxFree(a) safefree(a)
#define mexPrintf printf
#endif

#define EPS (1e-30) /* very small number */ 

/* 'x' is the input vector of the length 'T', 'a' is the output vector,
   'n' is the desired model order
*/
void burg( double *x, double *a, int n, int T)
{
  double   *ef,*eb,*efold,*ebold ;
  double   *aa,*rc;

  double num, den ;
  int t, ik ;

  SAFEMALLOC(ef,T,1) 
  SAFEMALLOC(eb,T,1) 
  SAFEMALLOC(efold,T,1) 
  SAFEMALLOC(ebold,T,1) 
  SAFEMALLOC(aa,n,1) 
  SAFEMALLOC(rc,T,1) 

  memmove(ef,x,T*sizeof(double)) ;
  memmove(eb,x,T*sizeof(double)) ;

  /* inic. vektoru dopredne a zpetne chyby predikce */

   for (ik=0;ik<n;ik++)  {
   /* vypocet parcialni korelace a koeficientu odrazu v sekci ik */
     num = 0.0;
     den = 0.0;

     for (t=ik+1;t<T;t++)
      { den += ef[t]*ef[t] + eb[t-1] * eb[t-1];
      	num += ef[t] * eb[t-1]; }

     num *= 2.0;

     if (den>EPS) rc[ik]=-(a[ik]=num/den);
             else rc[ik]=a[ik]=0.0 ;

     /* filtrace chybovych signalu */
     memmove(efold,ef,T*sizeof(double)) ;
     memmove(ebold,eb,T*sizeof(double)) ;

     for (t=1;t<T;t++)  {
        ef[t]=ef[t] +(rc[ik] * ebold[t-1]);
        eb[t]=ebold[t-1] + (rc[ik] * efold[t]);
        }

     if (ik>0)
        for(t=0;t<ik;t++ )
          a[t]=aa[t]+rc[ik]*aa[ik-t-1];

     memmove(aa,a,sizeof(double)*(ik+1)) ;
     }

   mxFree(ef) ; mxFree(eb) ; mxFree(efold) ; mxFree(ebold) ;
   mxFree(aa) ; mxFree(rc) ;
}

