/* This is the main source file for program cepdet, 
   performing cepstral based speech detection on 16-bit, 8kHz, 
   mono raw sound files.
   
   Jan Kybic, February 1996
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "cepstr.h"

#define WSTEP (CEPWLEN/2)     /* window shift step size */
#define K 0.2              /* adaptation constant for cepstr. coefs. */
#define L 0.05		   /* adaptation constant for threshold params. */
#define REST 20		   /* how many windows are guaranteed to be voiceless */

#define INT short int


static INT ibuf[WSTEP],*ip ; /* integer input buffer */
float x[CEPWLEN],han[CEPWLEN] ; /* one window of input signal */

float a[CEPN],ci[CEPN],c0[CEPN] ;


static void eavg()
{ float *c0p,*cip ;
  
  for(c0p=c0,cip=ci;c0p<c0+CEPN;c0p++,cip++)
     *c0p=K*(*cip)+(1-K)*(*c0p) ;
}     

void main(int argc,char *argv[])
{ 
  float *fp,*hp ;
  static FILE *inf,*outf ;  
  float Dmean, Dmean2, Dvar, dist, threshold,alpha ;
  int i,r ;

  puts("cepdet version 1.0, cepstral distance based speech detection") ;
  
  if (argc!=3)
   { puts("usage: cepdet input_file.bin output_file.det") ;
     return ;
   }
   
  if ((inf=fopen(argv[1],"r"))==NULL)
   { printf("cepdet: !!! Cannot open %s for reading.\n",argv[1]) ;
     return ; }

  if ((outf=fopen(argv[2],"w"))==NULL)
   { printf("cepdet: !!! Cannot open %s for writing.\n",argv[2]) ;
     fclose(inf) ; return ; }

  i=0 ; Dmean=Dmean2=Dvar=threshold=0 ;
  for (fp=x;fp<x+CEPWLEN;fp++) *fp=0.0 ;
  initaceps(han) ;
  
  while (fread(ibuf,sizeof(INT),WSTEP,inf)==WSTEP)
    { 
      memmove(x,x+WSTEP,(CEPWLEN-WSTEP)*sizeof(float)) ;
      for(fp=x+CEPWLEN-WSTEP,hp=han,ip=ibuf;ip<ibuf+WSTEP;fp++,ip++,hp++) 
         *fp=(float)(*ip) * (*hp) ;
           
      r=0 ; i++ ;     
      burg(x,a,&alpha) ;
      a2c(a,ci,alpha) ;

      
      if (i==1) { memmove(c0,ci,CEPN*sizeof(float)) ;
                }
 else { de(ci,c0,&dist) ;
        if (i==2) { for(fp=c0,hp=ci;fp<c0+CEPN;fp++,hp++)
                     *fp=((*fp)+(*hp))/2 ;
                   eavg() ;
                  Dmean=dist ; Dmean2=dist*dist ;
                  threshold=Dmean+2*sqrt(Dvar) ;                                  
                }
         else if (i<REST || dist<threshold)
                { /* no voice */
                  eavg() ;
                  Dmean=(1-L)*Dmean+L*dist ;
                  Dmean2=(1-L)*Dmean2+L*dist*dist ;
                  Dvar=Dmean2-Dmean*Dmean ;
                  threshold=Dmean+2*sqrt(Dvar) ;
                }                     
         else /* voice */ r=1 ;
       }     
#if 1      
      printf("%8.3g %8.3g %8.3g %8.3g %8.3g %2d\n",dist,threshold,Dmean,Dmean2,Dvar,r) ;
#endif

 fprintf(outf,"%d\n",r) ;
    }
		      
  fclose(inf) ; fclose(outf) ;    
}     