/* This is the main source file for program cepdist, 
   calculating cepstral distance for 16-bit, 8kHz, mono raw sound files, 
   assuming clean signal and detector output are known. It prints the resulting
   mean global distance and mean distance for voiced parts. 

   For info, type cepdist with no arguments.
   
   Jan Kybic, February-April 1996
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "cepstr.h"
 
#define WSTEP (CEPWLEN/2) /* FFT window length */

#define K 0.2              /* adaptation constant for cepstr. coefs. */

#define INT short int

static INT ibufn[WSTEP],ibufc[WSTEP],*ip ;
float xn[CEPWLEN],xc[CEPWLEN],han[CEPWLEN] ;

float a[CEPN],cc[CEPN],cn[CEPN] ;

void main(int argc,char *argv[])
{ 
  float *fp,*hp ;
  float dist,cpdt,cpdv,alpha ;  
  double distDouble ;
  
  int det ; 		/* current detector output */
  FILE *infn,*infc,*infd,*outfs ;  
  int countt,countv ; /* count of all and voiced windows */

  puts("cepdist version 1.0, cepstral distance calculation") ;
  
  if (argc!=4 && argc!=5)
   { puts("usage: cepdist noisy_sig.bin clean_sig.bin voice.det [cdist.dat]") ;
     puts(" noisy_sig ... measured signal") ;
     puts(" clean_sig ... corresponding clean signal") ;
     puts(" voice.det ... detector output") ; 
     puts(" cdist.dat   ... cepstral distance for each segment, binary form ") ;
     puts("Signals are binary, mono, 16bit, signed, host ordering.") ;
     puts("We assume 256 samples/window, 128 samples frame shift") ;

     return ;
   }
   
  if ((infn=fopen(argv[1],"r"))==NULL)
   { printf("cepdist: !!! Cannot open %s for reading.\n",argv[1]) ;
     return ; }

  if ((infc=fopen(argv[2],"r"))==NULL)
   { printf("cepdist: !!! Cannot open %s for reading.\n",argv[2]) ;
     return ; }

  if ((infd=fopen(argv[3],"r"))==NULL)
   { printf("cepdist: !!! Cannot open %s for reading.\n",argv[3]) ;
     return ; }

  if (argc==4) outfs=NULL ;
   else
  if ((outfs=fopen(argv[4],"w"))==NULL)
   { printf("snr: !!! Cannot open %s for writing.\n",argv[4]) ;
     return ; }

  cpdt=cpdv=0.0 ; countt=countv=0 ;
  for (fp=xn;fp<xn+CEPWLEN;fp++) *fp=0.0 ;
  for (fp=xc;fp<xc+CEPWLEN;fp++) *fp=0.0 ;
  initaceps(han) ;

  /* read all windows */
  while (fread(ibufn,sizeof(INT),WSTEP,infn)==WSTEP)
    { 
      if (fread(ibufc,sizeof(INT),WSTEP,infc)!=WSTEP)
        { puts("cepdist: !!! Cannot read clean signal") ;
          return ;
        }

      if (fscanf(infd,"%u",&det)!=1)
        { puts("cepdist: !!! Warning - detector output too short") ;
          det=0 ;
        }
      
      if (det!=0 && det!=1) 
        { puts("cepdist: !!! Unexpected detector output (not 0 or 1)") ;
          return ;
        }
      memmove(xn,xn+WSTEP,(CEPWLEN-WSTEP)*sizeof(float)) ;
      for(fp=xn+CEPWLEN-WSTEP,hp=han,ip=ibufn;ip<ibufn+WSTEP;fp++,ip++,hp++) 
         *fp=(float)(*ip) * (*hp) ;
      memmove(xc,xc+WSTEP,(CEPWLEN-WSTEP)*sizeof(float)) ;
      for(fp=xc+CEPWLEN-WSTEP,hp=han,ip=ibufc;ip<ibufc+WSTEP;fp++,ip++,hp++) 
         *fp=(float)(*ip) * (*hp) ;

      burg(xc,a,&alpha) ;
      a2c(a,cc,alpha) ;
      burg(xn,a,&alpha) ;
      a2c(a,cn,alpha) ;
      de(cc,cn,&dist) ; /* calculate distance */

      distDouble=(double)dist ;
      if (outfs) fwrite(&distDouble,sizeof(double),1,outfs) ;
              
      countt++ ; cpdt+=dist ;
      if (det) { countv++ ; cpdv+=dist ; }
		      
    }
  if (countt>0) printf("global cdist=%.1f  ",cpdt/countt) ;
     else { puts("Signal too short, cdist not available") ;
            return ; }

  if (countv>0) printf("voiced segments cpdt=%.1f\n",cpdv/countv) ;
     else { puts(" no voiced segments") ;
          }
  fprintf(stderr,"%.1f : %.1f\n",cpdt/countt,cpdv/countv) ;           		      
		      
  fclose(infn) ; fclose(infc) ; fclose(infd) ;  fclose(outfs) ;  
}     