/* This is a source file for cepstr module */
/* It implements operations on cepstral coefficients */
/* by P.P and J.K. */

#include <math.h>
#include <string.h>
#include "cepstr.h"

/*************************************************************************
 |        Funkce  pro vypocet autoregresnich koeficientu Burgovym         |
 |        algoritmem.                                                     |
 |                                                                        |
 |        !!! POZOR 'p' = pocet a-koeficientu bez    a[0]=1 !!!           |
 |                                                                        |
 |        'delka' = delka okna se vstupnim signalem                       |
 |                                                                        |
 |        Algoritmus je upraven podle Matlabu, puvodni neposkytoval zcela |
 |        shodne vysledky.                                                |
 ------------------------------------------------------------------------ */

int burg( float *x, float *a, float *alfa)
{

  static float   ef[CEPWLEN],eb[CEPWLEN],efold[CEPWLEN],ebold[CEPWLEN] ;
  static float   aa[CEPN],rc[CEPN];
 
  float pom, num, den ;
  int t, ik ;

   memmove(ef,x,CEPWLEN*sizeof(float)) ;
   memmove(eb,x,CEPWLEN*sizeof(float)) ;

  /* inic. vektoru dopredne a zpetne chyby predikce */

   *alfa = 0.0;
   for (t=0;t<CEPWLEN;t++) *alfa += ef[t]*ef[t];
   *alfa /=CEPWLEN;


   for (ik=1;ik<CEPN;ik++)  {
   /* vypocet parcialni korelace a koeficientu odrazu v sekci ik */
     num = 0.0;
     den = 0.0;

     for (t=ik;t<CEPWLEN;t++)  
      { den += ef[t]*ef[t] + eb[t-1] * eb[t-1];
      	num += ef[t] * eb[t-1]; }

     num *= 2.0;

     if (den==0) return(-2);
     
     rc[ik]=a[ik]=-num/den;

     /* filtrace chybovych signalu */
     memmove(efold,ef,CEPWLEN*sizeof(float)) ;
     memmove(ebold,eb,CEPWLEN*sizeof(float)) ;

     for (t=1;t<CEPWLEN;t++)  {
        ef[t]=ef[t] +(rc[ik] * ebold[t-1]);
        eb[t]=ebold[t-1] + (rc[ik] * efold[t]);
        }

     pom = 1 - rc[ik] * rc[ik];
     *alfa *= pom ;

     if (ik>1)
        for(t=1;t<ik;t++ )
          a[t]=aa[t]+rc[ik]*aa[ik-t];

     memmove(aa+1,a+1,sizeof(float)*ik) ;
     }
     
   a[0]=1.0;
   return (0);
}


void a2c(float *a,float *c,float alpha)
{ int n,k ; float sum ;

  for(n=1;n<CEPN;n++)
  { sum=0;
    for(k=1;k<n;k++) sum=sum+(n-k)*c[n-k]*a[k];
    c[n]=-a[n]-sum/n;
  }
  c[0]=(float) log((double)alpha) ;
}


void initaceps(float *han)
{ int i ;
  for(i=0;i<CEPWLEN;i++)
    han[i]=.5*(1-cos(2*PI*i/CEPWLEN)) ;
}     


/**************************************************************************
 |         Funkce  pro vypocet kepstralni vzdalenosti.                    |
	**************************************************************************/

int de( float *c1, float *c2, float *D)
{
	double sum ;
	int n ;

	for ( n=1, sum=0 ; n < CEPN ; n++ )
		 sum += ( c1[n] - c2[n] ) * ( c1[n] - c2[n] ) ;

	*D = 4.3429 * sqrt(2*sum); 

	return(0);
}	

