/* This is the implementation of the Kalman filter operating on a */
/* system consisting of two AR models, exactly as in darkalm.m   */
/* except that it does not calculate prediction error */
/* This version uses Cholesky decomposition */
/* Jan Kybic */


#ifndef lint
static char vcid[] = "$Id$";
#endif /* lint */

#ifndef MAKING_MEX
#include <stdlib.h>
#include <string.h>

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=malloc(a*b*sizeof(double)))==NULL) \
{ fprintf("DARKALM3C: could not malloc") ; exit(1) ; }
#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=malloc(a*b*sizeof(int)))==NULL) \
{ fprintf("DARKALM3C: could not malloc") ; exit(1) ; }
#define mxFree(a) free(a)
#define mexPrintf printf
#endif

#include "ludecomp.c" /* the LU decomposition routines */

#define ABS(x) ((x)>0 ? (x) : -(x))

static void darkalmanc(double *yp,double *zp,double *asp,
   double *bsp,double *csp,
   double *anp,double *bnp,double *cnp,int T,int ns, int nbs, int ncs, 
   int nn, int nbn, int ncn, double r)
{ int i,j,k,n ;
  /* values */
  double qs,qn,e,tmp,tmp1,rat ;
  /* buffers */
  double *P1,*P2,*xfbp,*xpbp,*Lp,*LLp,*ypb,*Fb,*PTp ;
  /* just pointers into buffers */
  double *xfp,*xpp,*Fp,*work ;
  /* some other pointers */
  double *Prow,*Pel ;
  int do_backward_run=1 ; /* a flag */
  int *indx ;

  n=ns+nn ; /* total order of the system */

  SAFEMALLOC(P1,n,n)
  SAFEMALLOC(P2,n,n) 
  SAFEMALLOC(xpbp,n,T+1)
  SAFEMALLOC(xfbp,n,T)
  SAFEMALLOC(Lp,1,n)
  SAFEMALLOC(LLp,1,n)
  SAFEMALLOC(Fb,n,T*n)
  SAFEMALLOC(PTp,n,n)
  SAFEMALLOC(work,n,1) 
  SAFEMALLOCINT(indx,n,1) ;

  qn=(*bnp)*(*bnp) ; /* noise subsystem input variance */
  qs=(*bsp)*(*bsp) ; /* signal subsystem input variance */

  /* set the pointers to the beginning*/
  xfp=xfbp ; xpp=xpbp ; /* filtered and predicted state estimates */  
  ypb=yp ; Fp=Fb ;

  /* initialize state */
  for(i=0;i<n;i++) xpp[i]=0.0 ;

  /* initialize matrix P */
  for(i=0;i<n;i++)
    for(j=0;j<n;j++) P1[i*n+j]=0.0 ;

  for(i=0,Prow=P1 ;i<nn;i++, Prow+=n) 
    { for(j=0;j<i;j++) Prow[j]=cnp[i-j] ;
      for(;j<nn;j++)   Prow[j]=cnp[j-i] ;
    }
  for(i=0,Prow+=nn;i<ns;i++, Prow+=n)
    { for(j=0;j<i;j++) Prow[j]=csp[i-j] ;
      for(;j<ns;j++)   Prow[j]=csp[j-i] ;
    }
  
  /* here begins the forward run */
  for(k=0;k<T;k++) {
    /* data step */
    /* calculate and store prediction error */
    e=*zp++ - (xpp[n-1]+xpp[nn-1]) ; 
    rat=1/(P1[(n-1)*n+(n-1)]+P1[(nn-1)*n+nn-1]+2*P1[(nn-1)*n+n-1]+r) ;

    /* calculate Kalman gain L and correct state estimate */
    tmp1=e*rat ;
    for(i=0;i<n;i++) { 
      LLp[i]=tmp=P1[i*n+n-1]+P1[i*n+nn-1] ; Lp[i]=rat * tmp ;
      xfp[i]=xpp[i]+tmp*tmp1 ;
    }

    /* update covariance matrix */
    for(i=0;i<n;i++)
      for(j=0;j<n;j++) P1[n*i+j]-=Lp[i]*LLp[j] ;

    /* time step */
    xpp+=n ;  xfp+=n ; /* shift pointers */

    /* advance state vector */
    for(tmp=0.0,j=0;j<ns;j++) tmp+=asp[ns-j-1]*xfp[-n+nn+j] ;
    for(j=0;j<ns-1;j++) xpp[nn+j]=xfp[nn+1-n+j] ;
    xpp[n-1]=tmp ;
    for(tmp=0.0,j=0;j<nn;j++) tmp+=anp[nn-j-1]*xfp[-n+j] ;
    for(j=0;j<nn-1;j++) 
        xpp[j]=xfp[1-n+j] ; 
    xpp[nn-1]=tmp ;

    /* advance the covariance matrix as P=A*P*A'+Q */
    for(i=0;i<n;i++) { /* i is now the row index */
      for(tmp=0.0,j=0;j<ns;j++) tmp+=asp[ns-j-1]*P1[i*n+(nn+j)] ;
      for(j=0;j<ns-1;j++) P2[i*n+(nn+j)]=P1[i*n+(nn+j+1)] ;
      P2[i*n+(n-1)]=tmp ;
      for(tmp=0.0,j=0;j<nn;j++) tmp+=anp[nn-j-1]*P1[i*n+j] ;
      for(j=0;j<nn-1;j++) P2[i*n+j]=P1[i*n+(j+1)] ;
      P2[i*n+(nn-1)]=tmp ;
    }
    /* now P2=P1*A' */ 
    for(i=0;i<n;i++) { /* i is now the column index */
      for(tmp=0.0,j=0;j<ns;j++) tmp+=asp[ns-j-1]*P2[i+(nn+j)*n] ;
      for(j=0;j<ns-1;j++) P1[i+(nn+j)*n]=P2[i+(nn+j+1)*n] ;
      P1[i+(n-1)*n]=tmp ;
      for(tmp=0.0,j=0;j<nn;j++) tmp+=anp[nn-j-1]*P2[i+j*n] ;
      for(j=0;j<nn-1;j++) P1[i+j*n]=P2[i+(j+1)*n] ;
      P1[i+(nn-1)*n]=tmp ;
    }
    /* now P1=A*P2=A*P1old*A' */
    P1[(n-1)*n+(n-1)]+=qs ;
    P1[(nn-1)*n+(nn-1)]+=qn ;
    /* now P1=A*P1old*A'+Q */

    if (!do_backward_run) continue ;

    /* proceed to calculating F */
    /* first of all, move P1 transposed to PTp */
    for(i=0;i<n;i++)
      for(j=0;j<n;j++) PTp[j*n+i]=P1[i*n+j] ;
    /* LU decompose */
    if (ludcmp(PTp,n,indx,work)!=0) /* singularity detected */
      { mexPrintf("DARKALMC: Singular covariance matrix.\n") ;
        do_backward_run=0 ; continue ; 
      }

    /* and backsubstitute while transposing as needed */
    for(i=0;i<n;i++) {
      for(j=0;j<n;j++) Fp[i*n+j]=P2[i*n+j] ; 
      lubksb(PTp,n,indx,Fp+i*n) ;
    }

    Fp+=n*n ; /* advance the pointer */
  } 
  /* end of the forward run */

  if (do_backward_run) { 
    xfp-=n ; yp=ypb+(T-1) ; Fp-=n*n ;
    for(i=0;i<n;i++) work[i]=xpp[i] ;
    /* backward run */
    for(k=T-1;k>=0;k--) {
      for(i=0;i<n;i++) work[i]-=xpp[i] ;
      for(i=0;i<n;i++)
	for(j=0;j<n;j++) xfp[i]+=Fp[i*n+j]*work[j] ;
      for(i=0;i<n;i++) work[i]=xfp[i] ; 
    Fp-=n*n ; xfp-=n ; xpp-=n ;
    }
  /* end of the backward run */
  }

  
  for(i=0;i<T;i++) ypb[i]=xfbp[n-1+i*n] ;
      
  mxFree(P1) ; mxFree(P2) ; 
  mxFree(xpbp) ; mxFree(xfbp) ; mxFree(Lp) ; 
  mxFree(LLp) ; mxFree(Fb) ; mxFree(PTp) ; mxFree(work) ; mxFree(indx) ;
}
