/* This is a header file for 'C' implemetations */
/* Noisy speech enhancement algorithm described by gerhard Doblinger */
/* Jan Kybic */

#ifndef DOBL_H
#define DOBL_H


/* readftype is a type for function to be called from within dobl_do
   to get a chunk of input data. The function should store them as floats
   into an area pointed to by p. It should return 1 if there is no more
   data available, negative number on errors and 0 otherwise. */

#ifndef READFTYPEDEF
#define READFTYPEDEF    
typedef int readftype(float *p) ;
#endif
    

/* dobl_init initializes the module. f should point to the function to
   be called to obtain data. dobl_init sets *sizep to the size of chunks
   it wishes to communicate in, usually a power of 2 nad *outbuf to the address
   of an output buffer. Caller must not change it! 
   It returns 0 if no error occured. */

int dobl_init(readftype *f,int *sizep,float **outbuf,void *) ;

int dobl_close() ; /* clean up */

/* dobl_do attempts to perform the desired noise reduction and stores
   the next *sizep output samples into an area pointed to by outbuf.
   Return values are the same as for the readftype *f function */
    
int dobl_do() ;

#endif
