/* This is the main source file for program enedet, 
   performing energy based speech detection on 16-bit, 8kHz, mono raw sound files.
   
   Jan Kybic, February 1996
*/

#include <stdio.h>
 
#define WLEN 256      /* FFT window length */
#define POWER 8       /* 2^POWER=WLEN */
#define WSTEP 128     /* FFT window shift step size */
#define THRESHOLD 1.5 
#define K 0.2         /* adaptation constant */
#define REST 50       /* this many windows are guaranteed to be voiceless */

#define INT short int
#define MAXINT 32767


static INT ibuf[WSTEP],*ip ; /* integer input buffer */

void main(int argc,char *argv[])
{ 
  float energy,e,e1,e2 ;
  static FILE *inf,*outf ;  
  int i ;

  puts("enedet version 1.0, energy based speech detection") ;
  
  if (argc!=3)
   { puts("usage: enedet input_file.bin output_file.det") ;
     return ;
   }
   
  if ((inf=fopen(argv[1],"r"))==NULL)
   { printf("enedet: !!! Cannot open %s for reading.\n",argv[1]) ;
     return ; }

  if ((outf=fopen(argv[2],"w"))==NULL)
   { printf("enedet: !!! Cannot open %s for writing.\n",argv[2]) ;
     fclose(inf) ; return ; }

  energy=-1 ; e1=0 ; i=0 ;
  while (fread(ibuf,sizeof(INT),WSTEP,inf)==WSTEP)
    { 
      for(e2=0,ip=ibuf;ip<ibuf+WSTEP;ip++) e2+=(*ip) * (*ip) ;
      e=e1+e2 ; e1=e2 ;
      
      i++ ;
		
      if (energy<0) { energy=e ; 
                      fputs("0\n",outf) ;
                      continue ; 
                    }
      if (i>REST && e>THRESHOLD*energy) fputs("1\n",outf) ;
                      else  { fputs("0\n",outf) ;
			      energy=(1-K)*energy+K*e ;
		            }
		      
    }
		      
  fclose(inf) ; fclose(outf) ;    
}     