/* FFT                                         */
/* Jan Kybic                                   */
/* see fft.h				       */


#include<stdio.h>
#include<math.h>
#include<stdlib.h>

#include "fft.h"


static complex *work[2],*workbuff,*w ;
static unsigned length,power ;


int initfft(int p)
{ unsigned k ;
  float o ;
  
  power=p ; length=1<<p ;

  if (!(workbuff=malloc(length*sizeof(complex))))  return(-1) ;
  if (!(w=malloc(length*sizeof(complex))))         return(-1) ;

  o=-4*asin(1)/length ;
  for(k=0;k<length;k++)
    { (w+k)->r=cos(o*k) ; (w+k)->i=sin(o*k) ; }
  return(0) ;  
}

void closefft()
{  free(w) ; free(workbuff) ; }

/* ---------------------------------------- */

/* There are two implementations, the first one uses three nested loops,
   while the second uses only one two but uses many bitwise logical
   operations. Either may run faster, depending on your machine. */

#define PREFERE_LOOPS
#ifdef PREFERE_LOOPS

void fft(complex *in,complex *out)
{ int initialpage ;
  complex *inbuff,*outbuff,*inptr1,*inptr2,*inf1,*inf2,*outf,*outptr ;
  float r1,r2,i1,i2 ;
  unsigned step,num,half,i,j,hlength ;

  work[0]=out ; work[1]=workbuff ;
  initialpage=(power & 1) ^1 ;
  inbuff=in ; outbuff=work[initialpage] ;

  step=1 ; num=length ; hlength=length>>1 ;
  while(num>1)
   { half=num/2 ;
     inptr1=inbuff ; inptr2=inbuff+hlength ;
     outptr=outbuff ;
     for(j=0;j<step;j++,inptr1++,inptr2++,outptr++)
      { inf1=inptr1 ; inf2=inptr2 ; outf=outptr ;
	for(i=0;i<half;i++)
	 { outf->r=inf1->r+inf2->r ;
	   outf->i=inf1->i+inf2->i ;
	   r1=inf1->r - inf2->r ; i1=inf1->i - inf2->i ;
	   r2=w[i*step].r ; i2=w[i*step].i ;
	   outf+=step ;
	   outf->r=r1*r2-i1*i2 ;
	   outf->i=r1*i2+r2*i1 ;
	   outf+=step ;
	   inf1+=step ;
	   inf2+=step ;
	 }
      }
     num=half ; step<<=1 ;
     inbuff=work[initialpage] ; outbuff=work[initialpage^=1] ;
   }
}

#else

void fft(complex *in,complex *out)
{ char initialpage ;
  complex *inbuff,*outbuff,*inptr1,*inptr2,*outptr ;
  float r1,r2,i1,i2 ;
  register unsigned k,step,mask ;
  unsigned hlength,len ;

  work[0]=out ; work[1]=workbuff ;
  mask=0 ; len=length ;
  initialpage=(power & 1) ^1 ;
  inbuff=in ; outbuff=work[initialpage] ;

  step=1 ; hlength=len>>1 ;

  while(step<len)
   { inptr1=inbuff ; inptr2=inbuff+hlength ;

     for(k=0;k<hlength;k++,inptr1++,inptr2++)
       { (outptr=outbuff+((k & mask ) | ((k & ~mask) << 1)))
           ->r=inptr1->r + inptr2->r ;
	 outptr->i=inptr1->i + inptr2->i ;
	 r1=inptr1->r - inptr2->r ; i1=inptr1->i - inptr2->i ;
	 r2=w[k & (~mask)].r ; i2=w[k & (~mask)].i ;
	 (outptr+=step)->r=r1*r2 - i1*i2 ;
	 (outptr)->i=r1*i2 + r2*i1 ;
       }
     mask|=step ; step<<=1 ;
     inbuff=work[initialpage] ; outbuff=work[initialpage^=1] ;
   }
}

#endif

/* End of fft.c */
