/* This is a secondary source file 'kalmseflt.c' for  
   Kalman filter based speech enhancement program 'kalmse'

   Jan Kybic, October 1997
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "kalmse.h"


/* ------------------------------------------------------------------------ */

void filter_init(kl_prms *p)
{ int i,j,r ;

  p->wllog=(int)(log10(p->wl)/log10(2)) ;
  rprintf(DBGL,"Window length is 2^%d.\n",p->wllog) ;

  if (initfft(p->wllog)!=0) {
    eprintf(ERRL,"Not enough memory in 'initfft'.\n") ;
    exit(-1) ;
  }

  p->hann=safemalloc(sizeof(double)*p->wl) ;
  p->buf=safemalloc(sizeof(double)*p->wl) ;
  p->sps=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->ssps=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->circb=safemalloc(sizeof(double) * (p->wl/2+1) * p->lb) ;
  p->cbmin=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->lsmin=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->nps=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->snps=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->cbuf1=safemalloc(sizeof(complex)*p->wl) ;  
  p->cbuf2=safemalloc(sizeof(complex)*p->wl) ;
  p->an=safemalloc(sizeof(double)*p->nn) ;
  p->as=safemalloc(sizeof(double)*p->ns) ;
  p->sss=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->cs=safemalloc(sizeof(double)*(p->wl/2+1)) ;
  p->cn=safemalloc(sizeof(double)*(p->wl/2+1)) ;
#ifdef KALMSEPVM
  p->tids=safemalloc(sizeof(int)*p->pv) ;
  p->si=safemalloc(sizeof(struct slinfo)*p->pv) ;
#endif

  /* Make Hanning window with unit average */
  for(r=0;r<p->wl;r++) 
    p->hann[r]=(1-cos(2*M_PI*(r+1)/(p->wl+1))) ;

  /* initialize minima tracking and smoothing */
  p->cbi=0 ; p->lsc=p->la ;
  for(i=0;i<(p->wl/2+1);i++) {
    p->lsmin[i]=p->cbmin[i]=p->sps[i]=
      p->snps[i]=p->ssps[i]=p->nps[i]=0.0 ;
    for(j=0;j<p->lb;j++) p->circb[j*(p->wl/2+1)+i]=0.0 ;
  }
  /* initialize output buffer */
  for(i=0;i<p->buflen;i++) p->outbuf[i]=0.0 ;  

#ifdef KALMSEPVM
  /* initialize slave info structures */
  for(i=0;i<p->pv;i++) p->si[i].ws=0 ;
#endif

}

/* ------------------------------------------------------------------------ */

void filter_close(kl_prms *p)
{
  closefft() ;
  safefree(p->hann) ;  
  safefree(p->sps) ;
  safefree(p->ssps) ;
  safefree(p->circb) ;
  safefree(p->cbmin) ;
  safefree(p->lsmin) ;
  safefree(p->nps) ;
  safefree(p->snps) ;
  safefree(p->cbuf1) ;
  safefree(p->cbuf2) ;
  safefree(p->buf) ;
  safefree(p->an) ;
  safefree(p->as) ;
  safefree(p->sss) ;
  safefree(p->cs) ;
  safefree(p->cn) ;
#ifdef KALMSEPVM
  safefree(p->tids) ;
  safefree(p->si) ;
#endif
}

/* ------------------------------------------------------------------------ */

#define MEM_ORDER (16) /* MEM estimate order */

void calculate_power_spectrum(kl_prms *p,int ind) 
{ int i,j ;
  double a[MEM_ORDER],b,s1,s2,v
 ;

  switch(p->fm) {
  case 0: /* MEM method */
    burg(p->inbuf+ind,a,MEM_ORDER,p->wl) ;
    /* Now calculate the b coefficient - first get the spectrum that we
     would have if b were 1 */
    p->cbuf1[0].r=1.0 ; p->cbuf1[0].i=0.0 ;
    for(i=0;i<MEM_ORDER;i++) {
      p->cbuf1[i+1].r=-a[i] ; p->cbuf1[i+1].i=0.0 ;
    }
    for(i++;i<p->wl;i++) p->cbuf1[i].r=p->cbuf1[i].i=0.0 ;
    fft(p->cbuf1,p->cbuf2) ;
    s1=0.0 ;
    /* Compare the energies and calculate b so that they match */
    for(i=0;i<p->wl;i++) {
      v=p->cbuf2[i].r*p->cbuf2[i].r+p->cbuf2[i].i*p->cbuf2[i].i ;
      /* prevent division by zero */
      if (v==0.0) v=EPS ;
      s1+=(p->buf[i]=1/v) ;
    }
    if (s1==0.0) s1=EPS ;
    s2=0.0 ;
    for(i=0;i<p->wl;i++) s2+= p->inbuf[ind+i] * p->inbuf[ind+i] ;
    b=sqrt(s2/s1) ;
    for(i=0;i<p->wl/2+1;i++) p->buf[i]*=b ;
    return ;

  case 1: /* periodogram */
    for(j=0;j<p->wl;j++) {
      p->cbuf1[j].r=p->inbuf[j+ind]*p->hann[j] ;
      p->cbuf1[j].i=0.0 ;
    }
    fft(p->cbuf1,p->cbuf2) ;
    for(j=0;j<(p->wl/2+1);j++) 
      p->buf[j]=p->cbuf2[j].r*p->cbuf2[j].r+p->cbuf2[j].i*p->cbuf2[j].i ;
    return ;
  default:
    eprintf(ERRL,
"Invalid spectrum estimation method number in 'calculate_power_spectrum'.\n") ;
    exit(-1) ;
    }
    /* the power spectrum is now in buf */    
}

/* ------------------------------------------------------------------------ */

void  smooth_power_spectra(kl_prms *p)
{ int j ;

  for(j=0;j<(p->wl/2+1);j++) {
    p->sps[j]=p->alpha*p->sps[j]+(1-p->alpha)*p->buf[j] ;
    p->ssps[j]=p->alphas*p->ssps[j]+(1-p->alphas)*p->buf[j] ;
  }
}

/* ------------------------------------------------------------------------ */
void estimate_noise(kl_prms *p)
{ int i,j ; double v ;

  if (p->lsc >= p->la) {
    /* store the minimum of the last segment into the circular buffer */
    memcpy(p->circb+p->cbi*(p->wl/2+1),p->lsmin,(p->wl/2+1)*sizeof(double)) ;
    if (++p->cbi >= p->lb) p->cbi=0 ;
    /* store minimum of circular buffer to cbmin */
    for(i=0;i<(p->wl/2+1);i++) {
      p->cbmin[i]=p->circb[i] ;
      for(j=1;j<p->lb;j++) 
	if ((v=p->circb[j*(p->wl/2+1)+i])<p->cbmin[i]) p->cbmin[i]=v ;
      p->lsmin[i]=p->sps[i] ;
      p->nps[i]=p->eb * MIN(p->lsmin[i],p->cbmin[i]) ;
      p->snps[i]=p->be * p->snps[i] + (1-p->be) * p->nps[i] ; 
    }
    p->lsc=1 ;
  } else {
    p->lsc++ ;
    for(i=0;i<(p->wl/2+1);i++) {
      p->lsmin[i]=MIN(p->sps[i],p->lsmin[i]) ;
      p->nps[i]=p->eb * MIN(p->lsmin[i],p->cbmin[i]) ;
      p->snps[i]=p->be * p->snps[i] + (1-p->be) * p->nps[i] ; 
    }
  }

  /* the smoothed noise power spectrum estimate is now in snps */
}

/* ------------------------------------------------------------------------ */

void subtraction_rule(kl_prms *p)
{ int i ; double v ;

  for(i=0;i<(p->wl/2+1);i++) {
    switch(p->sr) {
    case 0: /* simple subtraction rule */
      p->sss[i]=p->ssps[i]-p->snps[i] ;
      break ;
    case 1: /* half way rectification */
      v=p->ssps[i]-p->snps[i] ;
      p->sss[i]=MAX(0.0,v) ;
      break ;
    case 2: /* two way rectification */
      v=p->ssps[i]-p->snps[i] ;
      p->sss[i]=ABS(v) ;
      break ;
    case 3: /* Wiener */
      if (p->ssps[i]==0.0) p->sss[i]=0.0 ; 
      else {
	v=p->ssps[i]-p->snps[i] ;
	v=MAX(0.0,v) ;
	p->sss[i]=(v*v)/p->ssps[i] ;
      }
      break ;
    case 4: /* Martin */
      if (p->snps[i]==0.0 || p->ssps[i]==0.0) p->sss[i]=p->ssps[i] ;
      else {
	v=(p->ssps[i]-p->snps[i])/p->snps[i] ;
	if (v<1.0) v=1.0 ; 
	else if (v>100.0) v=100.0 ;
	v=p->mh-(p->mh-p->ml)/2.0*log10(v) ;
	v=1-sqrt(v*p->snps[i]/p->ssps[i]) ;
        if (v<0.0) p->sss[i]=0.0 ; else
	  p->sss[i]=v*v*p->ssps[i] ;
      }
      break ;
    default:
      eprintf(ERRL,
	      "Invalid subtraction rule number in 'subtraction_rule'.\n") ;
      exit(-1) ;
    }
  }
  /* The speech estimate is now in sss */
}

/* ------------------------------------------------------------------------ */
void estimate_AR_parameters(kl_prms *p,int n,double *a,
			    double *b, double *sp, double *c) 
{ int i,T ;
  double v,s1,s2 ;

  /* Calculate autocorrelation using FFT from the given spectrum */
  T=p->wl ;
  p->cbuf1[0].i=0.0 ; 
  p->cbuf1[0].r=sp[0]/(T*T) ;

  for(i=1;i<T/2+1;i++) {
    p->cbuf1[i].i=p->cbuf1[T-i].i=0.0 ;
    p->cbuf1[i].r=p->cbuf1[T-i].r=sp[i]/(T*T) ;
  }
  fft(p->cbuf1,p->cbuf2) ;
  for(i=0;i<T/2+1;i++) c[i]=p->cbuf2[i].r ;
  /* the autocorrelation is now in 'c' */

  /* calculate the a parameters */
  switch (p->im) {
  case 0: burg(c,a,n,T/2) ;
          break ;
  case 1: tstlsw(c,a,n,T/2,p->tsmax) ;
          break ;
  default:
    eprintf(ERRL,
"Invalid identification method in 'estimate_AR_parameters'.\n") ;
      exit(-1) ;
  }

  /* Now calculate the b coefficient - first get the spectrum that we
     would have if b were 1 */
  p->cbuf1[0].r=1.0 ; p->cbuf1[0].i=0.0 ;
  for(i=0;i<n;i++) {
    p->cbuf1[i+1].r=-a[i] ; p->cbuf1[i+1].i=0.0 ;
  }
  for(i++;i<T;i++) p->cbuf1[i].r=p->cbuf1[i].i=0.0 ;
  fft(p->cbuf1,p->cbuf2) ;

  /* Compare the two spectra and calculate b so that the energies match */
  s1=0.0 ;
  for(i=1;i<T/2+1;i++) {
    v=p->cbuf2[i].r*p->cbuf2[i].r+p->cbuf2[i].i*p->cbuf2[i].i ;
    /* prevent division by zero */
    if (v==0.0) v=EPS ;
    s1+=1/v ;
  }
  s1*=T ; 
  if (s1==0.0) s1=EPS ;

  s2=0.0 ;
  for(i=1;i<T/2+1;i++) s2+=sp[i] ;
  *b=sqrt(s2/s1) ;
}

/* ------------------------------------------------------------------------ */
void kalman_filter(kl_prms *p,int ind)
{ int r,i ; double v,maxinp,maxout ;

  if (p->kf<0 || p->kf>2) {
    eprintf(ERRL,
	    "Invalid Kalman filtering method in 'filter_buffer'.\n") ;
    exit(-1) ;
  }

  for(maxinp=0.0,i=0;i<p->wl;i++)
    if ((v=ABS(p->inbuf[ind+i]))>maxinp) maxinp=v ;

  r=0 ; maxout=0.0 ; /* to keep the compiler happy */
  if (p->kf<2) {
    r=darkalman3c(p->buf,p->inbuf+ind,p->as,&p->bs,p->cs,p->an,&p->bn,
		  p->cn,p->wl,p->ns,1,(p->wl/2+1),p->nn,1,(p->wl/2+1),p->mc,
		  p->kf==0) ;
    for(maxout=0.0,i=0;i<p->wl;i++)
      if ((v=ABS(p->buf[i]))>maxout) maxout=v ;
  }
  /* if the fast method failed or user requires it, use SR */
  if (p->kf==2 || r!=0 || maxout>MAXOVERSHOOT*maxinp) {
    rprintf(PROCL,"S") ;
    darkalmansc(p->buf,p->inbuf+ind,p->as,&p->bs,p->cs,p->an,&p->bn,
		p->cn,p->wl,p->ns,1,(p->wl/2+1),p->nn,1,(p->wl/2+1),p->mc) ;
  }
}

/* ------------------------------------------------------------------------ */

void assemble_window(kl_prms *p,int ind)
{ int i ;
  
  if (p->wo>1) {
    for(i=0;i<p->wl;i++)
      p->outbuf[i+ind]+=p->buf[i]*p->hann[i]/p->wo ;
  } else 
    for(i=0;i<p->wl;i++) p->outbuf[i+ind]=p->buf[i] ;
}

/* ------------------------------------------------------------------------ */

void filter_buffer(kl_prms *p)
{ int i,j,ind ;

  filter_init(p) ;
  
  /* process the frames one by one */
  for(i=0;i<p->nwin;i++) {
    rprintf(PROCL,"[%d]",i) ;
    
    ind=i*p->wshft ; /* beginning of the window */

    calculate_power_spectrum(p,ind) ; /* calculate the spectrum first */
    smooth_power_spectra(p) ;         /* calculate the smoothed estimates */
    estimate_noise(p) ;               /* use Martin's algorithm */

    if (i < p->la * p->lb)  /* startup phase - do not filter */ 
      for(j=0;j<p->wl;j++) p->buf[j]=0.0 ; 
    /* alternatively p->buf[j]=p->inbuf[ind+j] ; */
     
    else {
    subtraction_rule(p) ;
    estimate_AR_parameters(p,p->ns,p->as,&p->bs,p->sss,p->cs) ;
    estimate_AR_parameters(p,p->nn,p->an,&p->bn,p->snps,p->cn) ;

    /* We are ready to do Kalman filtering, finally */
    kalman_filter(p,ind) ;
    }
    
    assemble_window(p,ind) ;
  } /* end of for i */      

  filter_close(p) ;
}
/* ------------------------------------------------------------------------ */

#ifdef KALMSEPVM /* here starts the PVM specific part */


void be_slave(kl_prms *p)
{ int mytid,master,r,i ;
 
  eprintf(DBGL,"My TID=%x parent=%x ",mytid=pvm_mytid(),master=pvm_parent()) ;

  if (mytid<0 || master<0) {
    eprintf(ERRL,"Error starting PVM and connecting to master") ;
    pvm_exit() ;
    return ;
  }

  eprintf(DBGL,"I am a slave %d, my master is %d.\n",mytid,master) ;

  if (pvm_recv(master,0)<0) {
    eprintf(ERRL,"Error receiving message. Exiting.\n") ;
  finish:
    pvm_exit() ;
    exit(-1) ;
  }    
  
  pvm_upkint(&p->nn,1,1) ;
  pvm_upkint(&p->ns,1,1) ;
  pvm_upkfloat(&p->mc,1,1) ;
  pvm_upkint(&p->kf,1,1) ;
  pvm_upkint(&p->tsmax,1,1) ;
  pvm_upkint(&p->wl,1,1) ;
  
  filter_init(p) ;
  p->inbuf=safemalloc(p->wl*sizeof(double)) ;
  /* Here we process everything */
  for(;;) {
     if (pvm_recv(master,1)<0) {
       eprintf(ERRL,"Error receiving message. Exiting.\n") ;
     cleanup_and_finish:
       filter_close(p) ;
       safefree(p->inbuf) ;
       goto finish ;
     }
     pvm_upkint(&r,1,1) ; /* frame number */
     if (r<0) break ; /* finished */
     pvm_upkint(&i,1,1) ; /* serial number */
     pvm_upkdouble(p->sss,p->wl/2+1,1) ;
     pvm_upkdouble(p->snps,p->wl/2+1,1) ;
     pvm_upkdouble(p->inbuf,p->wl,1) ;

     /* process the given window */
     estimate_AR_parameters(p,p->ns,p->as,&p->bs,p->sss,p->cs) ;
     estimate_AR_parameters(p,p->nn,p->an,&p->bn,p->snps,p->cn) ;
     kalman_filter(p,0) ;

     /* send the message back */
     pvm_initsend(PvmDataDefault) ;
     pvm_pkint(&r,1,1) ;
     pvm_pkint(&i,1,1) ;
     pvm_pkdouble(p->buf,p->wl,1) ;
     if (pvm_send(master,2)<0) {
       eprintf(ERRL,"Error sending message. Exiting.\n") ; 
       goto cleanup_and_finish ;
     }
  } /* end of the for loop */

  safefree(p->inbuf) ;
  filter_close(p) ;
  pvm_exit() ;
}

/* ------------------------------------------------------------------------ */

void filter_buffer_master(kl_prms *p)
{ int i,j,r,ind,nslaves ;
  static char *slave_args[3]={SLAVEARGS,NULL} ;

  filter_init(p) ;
  p->pend=0 ;

  pvm_mytid() ; /* enroll in PVM */

  pvm_setopt(PvmRoute,PvmRouteDirect) ; /* direct routing should be faster */

  /* start slaves */
  nslaves=pvm_spawn(SLAVENAME,slave_args,0,NULL,p->pv,p->tids) ;
  
  eprintf(INFOL,"I am a master. %d slaves started.\n",nslaves) ;
  if (nslaves!=p->pv) {
    eprintf(ERRL,"Not all slaves started. Exiting.\n") ;
    /* kill the remaining */
  finish:
    for(i=0;i<nslaves;i++) pvm_kill(p->tids[i]) ;
    pvm_exit() ;
    filter_close(p) ;
    exit(-1) ;
  }

  /* send the first message */
  pvm_initsend(PvmDataDefault) ;
  pvm_pkint(&p->nn,1,1) ;
  pvm_pkint(&p->ns,1,1) ;
  pvm_pkfloat(&p->mc,1,1) ;
  pvm_pkint(&p->kf,1,1) ;
  pvm_pkint(&p->tsmax,1,1) ;
  pvm_pkint(&p->wl,1,1) ;

  if (pvm_mcast(p->tids,p->pv,0)<0) {
    eprintf(ERRL,"Error sending message. Exiting.\n") ; 
    goto finish ;
  }

  /* process the frames one by one */
  for(i=0;i<p->nwin;i++) {
    
    ind=i*p->wshft ; /* beginning of the window */

    calculate_power_spectrum(p,ind) ; /* calculate the spectrum first */
    smooth_power_spectra(p) ;         /* calculate the smoothed estimates */
    estimate_noise(p) ;               /* use Martin's algorithm */

    if (i < p->la * p->lb)  /* startup phase - do not filter */ 
      for(j=0;j<p->wl;j++) p->buf[j]=0.0 ; 
    /* alternatively p->buf[j]=p->inbuf[ind+j] ; */
    
    else {
      subtraction_rule(p) ;
      if (i-p->la*p->lb<NPENDWIND*p->pv) /* initial task distribution */
	give_work(p,i,i % p->pv) ;
      else {
	j=process_message(p) ;
	give_work(p,i,j) ;
      }
     } /* end of the 'if (i<p->la*p->lb)' else branch */
  } /* end of for i */      
 
  /* process pending messages */
  while(p->pend>0) process_message(p) ;

  /* stop slaves */
  i=-1 ;
  pvm_initsend(PvmDataDefault) ;
  pvm_pkint(&i,1,1) ;
  if (pvm_mcast(p->tids,p->pv,1)<0) {
    eprintf(ERRL,"Error sending message. Exiting.\n") ; 
    goto finish ;
  }
	
  pvm_exit() ;
  filter_close(p) ;
}

/* ------------------------------------------------------------------------ */

int process_message(kl_prms *p)
{ int i,j,ind ;

  rprintf(PROCL,"[r") ;
  if (pvm_recv(-1,2)<0) {
    eprintf(ERRL,"Error receiving message. Exiting.\n") ;
    for(i=0;i<p->pv;i++) pvm_kill(p->tids[i]) ;
    pvm_exit() ;
    filter_close(p) ;
    exit(-1) ;
  }
  pvm_upkint(&i,1,1) ;
  pvm_upkint(&j,1,1) ;
  pvm_upkdouble(p->buf,p->wl,1) ;
  assemble_window(p,i*p->wshft) ;
  p->pend-- ;
  rprintf(PROCL,"%d]",i) ;
  return j ;
}

/* ------------------------------------------------------------------------ */

void give_work(kl_prms *p,int i,int j)
{

  rprintf(PROCL,"[s%d",i) ;

  /* free task index is in j now */
  pvm_initsend(PvmDataDefault) ;
  pvm_pkint(&i,1,1) ;
  pvm_pkint(&j,1,1) ;
  pvm_pkdouble(p->sss,p->wl/2+1,1) ;
  pvm_pkdouble(p->snps,p->wl/2+1,1) ;
  pvm_pkdouble(p->inbuf+i*p->wshft,p->wl,1) ;
  if (pvm_send(p->tids[j],1)<0) {
    eprintf(ERRL,"Error sending message. Exiting.\n") ; 
    for(i=0;i<p->pv;i++) pvm_kill(p->tids[i]) ;
    pvm_exit() ;
    filter_close(p) ;
    exit(-1) ;
  }
  p->si[j].ws++ ;
  p->pend++ ;
  rprintf(PROCL,"]") ;
}

#endif /* ifdef KALMSEPVM */

/* ------------------ end of kalmseflt.c ------------------------------- */
