/*
  lusolve   The MEX implementation of linear equation set solver
  via LU decomposition based on implementation from Numerical Recipies 

	y=lusolve(A,b)

  Jan Kybic, October 1997

*/
#include <string.h>
#include "mex.h"

#include "../c/ludecomp.c"

#ifdef __GNUC__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#define myCreateMatrix(a,b) mxCreateFull(a,b,REAL)
#define myAlloc(a) mxCalloc(a,1)
#else /* hp version */
#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#endif

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt("DARKALMC: could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt("DARKALMC: could not malloc") ;

/* Input Arguments */

#define	A_IN	prhs[0]
#define	B_IN	prhs[1]

/* Output Arguments */

#define	Y_OUT	plhs[0]

#ifdef linux
void mexFunction(
                 int nlhs, Matrix *plhs[],
                 int nrhs, Matrix *prhs[]
		 )
#else
void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
#endif
{ int i,j,n ;

  double *yp,*Ap,*bp,*work,tmp,*T ;
  int *indx ;

  /* Check for proper number of arguments */
  
  if (nrhs != 2) {
    mexErrMsgTxt("LUSOLVE requires two input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt("LUSOLVE requires one output argument.");
  }
  
  if (!mxIsNumeric(prhs[0]) || mxIsComplex(prhs[0]) ||
      mxIsSparse(prhs[0]) || !mxIsDouble(prhs[0]))
    mexErrMsgTxt("LUSOLVE: Incorrect first argument type") ;

  n=mxGetM(prhs[0]) ;
  if (n!=mxGetN(prhs[0])) mexErrMsgTxt("LUSOLVE: A must be square") ;

  if (!mxIsNumeric(prhs[1]) || mxIsComplex(prhs[1]) ||
      mxIsSparse(prhs[1]) || !mxIsDouble(prhs[1]))
    mexErrMsgTxt("LUSOLVE: Incorrect second argument type") ;

  if (mxGetM(prhs[1])!=n || mxGetN(prhs[1])!=1)
    mexErrMsgTxt("LUSOLVE: Incorrect size of b.") ;

  
  Y_OUT = myCreateMatrix(n,1);

  yp = mxGetPr(Y_OUT); Ap=mxGetPr(A_IN) ; bp=mxGetPr(B_IN) ; 

  SAFEMALLOC(work,1,n)
  SAFEMALLOCINT(indx,1,n) 
  SAFEMALLOC(T,n,n)

  for(i=0;i<n;i++)
    for(j=0;j<n;j++) T[n*i+j]=Ap[n*j+i] ; 

  ludcmp(T,n,indx,work) ;
  for(i=0;i<n;i++) yp[i]=bp[i] ;
  lubksb(T,n,indx,yp) ;

  mxFree(work) ; mxFree(indx) ; mxFree(T) ;
  return;
}


#define MAKING_MEX

