/* This is a header file for 'C' implemetations */
/* Noisy speech enhancement algorithm described by Reiner Martin */
/* Jan Kybic */

#ifndef MARTIN_H
#define MARTIN_H


/* readftype is a type for function to be called from within martin_do
   to get a chunk of input data. The function should store them as floats
   into an area pointed to by p. It should return 1 if there is no more
   data available, negative number on errors and 0 otherwise. */

#ifndef READFTYPEDEF
#define READFTYPEDEF    
typedef int readftype(float *p) ;
#endif
    

/* martin_init initializes the module. f should point to the function to
   be called to obtain data. martin_init sets *sizep to the size of chunks
   it wishes to communicate in, usually a power of 2 nad *outbuf to the address
   of an output buffer. Caller must not change it! 
   It returns 0 if no error occured. */

int martin_init(readftype *f,int *sizep,float **outbuf,void *) ;

int martin_close() ; /* clean up */

/* martin_do attempts to perform the desired noise reduction and stores
   the next *sizep output samples into an area pointed to by outbuf.
   Return values are the same as for the readftype *f function */
    
int martin_do() ;

#endif