/* This is signal mixer */
/* by J.K. and P.P. */
/* 20.11 - added new switch -e */

#include <stdio.h>
#include <math.h>
#include <values.h>
#include <stdlib.h>

#define MINEN 1e-10   /* minimum energy per window */
#define MINRAT 1e-300 /* minimum ratio between signal and noise energy */


typedef short int INT ;


/* calculates sum of squares of WSTEP INT's long vector */
static double vectene(INT *p, INT length)
{ double e ;
/*  INT *q ;

  for(e=0,q=p;q<p+length;q++) e+=(double)(*q) * (double)(*q) ;*/

  INT j ;
  for ( j= 0, e=0 ; j<length; j++)
    e+=((double)p[j]*p[j]) ;
    
    
   return e ;
  
}

INT snr_comp ( FILE *infc, FILE *infn, FILE *infm, FILE *infd,
          double snr,
          INT wlen, INT wstep,
          unsigned long lengthc, INT extwin )
{
  double en1,en2,en ;   /* energy of noise signal */
  double ec1,ec2,ec ;   /* energy of clean voice signal */
  double sum ;         /* sum of energy ratios */
  double ratio ;
  double k ;           /* noise multiplicative constant */
  int det ;  		       /* current detector output */
  int j;
  INT *ibufn, *ibufc, *ibufm ;    /* pointers to integer buffers */
  int countv ;         /* count of voiced windows */
  unsigned long length ;  /* length of already processed signal */
  int pom1, pom2 ;

  if ( NULL == ( ibufc = calloc(wstep,sizeof(INT)) ) ) {
    fputs ( "mix: Memory allocation error!\n" , stderr) ;
    return -1 ;
    }
  if ( NULL == ( ibufn = calloc(wstep,sizeof(INT)) ) ) {
    fputs ( "mix: Memory allocation error!\n" ,stderr) ;
    free ( ibufc ) ;
    return -1 ;
    }
  if ( NULL == ( ibufm = calloc(wstep,sizeof(INT)) ) ) {
    fputs ( "mix: Memory allocation error!\n",stderr ) ;
    free ( ibufc ) ;      free ( ibufn ) ;
    return -1 ;
    }


  if (  (pom1=fread(ibufn,sizeof(INT),wstep,infn))!=wstep ||
        (pom2=fread(ibufc,sizeof(INT),wstep,infc))!=wstep )
        { fputs("mix: Cannot read from signal files.\n",stderr) ;
             return -1 ; }

  en1=vectene(ibufn,wstep) ;
  ec1=vectene(ibufc,wstep) ;

  countv=0 ;
  sum = 0 ;
  length = wstep ;


  /* read all windows */
  while ( length + wlen < lengthc )
    {

      pom1=fread(ibufn,sizeof(INT),wstep,infn) ;
      pom2=fread(ibufc,sizeof(INT),wstep,infc) ;
      if ( pom1 != wstep || pom2 != wstep )
        break ;

      if (fscanf(infd,"%i",&det)!=1)
        { fputs("mix: Warning! Detector output too short!\n",stderr) ;
          det=0 ;
        }

      if (det!=0 && det!=1)
        { fputs("mix: Unexpected detector output (not 0 or 1)!\n",stderr) ;
          return -1;
        }

      en2=vectene(ibufn,wstep) ;
      en=en1+en2 ;
      en1=en2 ;


      ec2=vectene(ibufc,wstep) ;
      ec=ec1+ec2 ;
      ec1=ec2 ;

      if (det==1) {

        ratio =  ec/en ;
  /*      if ( ratio < 1e-3 ) ratio=1e-3 ;*/
        sum += ( log10(ratio) ) ;
        countv++ ;
        }

      length += wstep ;
    }

  k=pow(10.0,(-1*(snr)/20.+sum/2/countv));
  
  fseek ( infc, 0, SEEK_SET ) ;
  fseek ( infn, 0, SEEK_SET ) ;

  while ( extwin-- && fread(ibufn,sizeof(INT),wstep,infn)==wstep)
    {
      for(j=0;j<wstep;j++)
        ibufm[j] = (INT) floor ((double)k*ibufn[j]) ;

      if (fwrite(ibufm,sizeof(INT),wstep,infm)!=wstep)
        { fputs("mix: Cannot write output signal!\n",stderr) ;
          return -1;
        }
    }

  while ( fread(ibufn,sizeof(INT),wstep,infn)==wstep &&
          fread(ibufc,sizeof(INT),wstep,infc)==wstep &&
          length + wlen < lengthc )
    {

      for(j=0;j<wstep;j++)
        ibufm[j] = (INT) floor ((double)ibufc[j]+k*ibufn[j]) ;

      if (fwrite(ibufm,sizeof(INT),wstep,infm)!=wstep)
        { fputs("mix: Cannot write output signal!\n",stderr) ;
          return -1;
        }

      length += wstep ;
    }

  free(ibufc); free(ibufn); free(ibufm);
  return 0;
}

/* ***************************************************************
                  Main block start here
**************************************************************** */

typedef struct
{ char *filename[4] ;
  INT wlen  ;
  INT wstep ;
  INT snr ;
  INT extwin ;
  unsigned long lengthc ;
} struct_param ;

void usage ( void )
{
      fputs ( "Usage: mix clean.bin noise.bin mx.bin clean.det            [Options]\n",stderr ) ;
      fputs ( "    clean.bin  -  clean signal file\n",stderr  ) ;
      fputs ( "    noise.bin  -  noise signal file\n",stderr  ) ;
      fputs ( "    mx.bin     -  mixture file\n",stderr  ) ;
      fputs ( "    clean.det  -  VAD detection file for clean speech\n",stderr  ) ;
      fputs ( "Optional parameters :                                     <Defaults>\n",stderr  ) ;
      fputs ( "    -l[INT]     - length of processed signal  <length of the signal>\n",stderr  ) ;
      fputs ( "    -w[INT]     - window length                                <256>\n",stderr  ) ;
      fputs ( "    -s[INT]     - window step                                  <128>\n",stderr  ) ;
      fputs ( "    -r[INT]     - desired (averaged) signal-to-noise ratio       <0>\n",stderr ) ;
      fputs ( "    -e[INT]     - number of extra speech-less windows to add     <0>\n",stderr ) ; 
}

INT param_set ( INT argc, char *argv[], struct_param *param )
{
	INT j, k ;

	for ( k = 0, j = 1 ; j < argc ; j++ )

		if ( argv[j][0] == '-' )
			switch ( argv[j][1] ) {
				case 'l' :
				case 'L' : param->lengthc = atol ( &argv[j][2] ) ;
									 break ;
				case 'w' :
				case 'W' : param->wlen = atoi ( &argv[j][2] ) ;
									 break ;
				case 's' :
				case 'S' : param->wstep = atoi ( &argv[j][2] ) ;
									 break ;
				case 'r' :
				case 'R' : param->snr = atoi ( &argv[j][2] ) ;
									 break ;
				case 'e' :
				case 'E' : param->extwin = atoi ( &argv[j][2] ) ;
									 break ;
				default  : usage () ;  return -1 ;
				}
		else
			if ( k < 4 )  param->filename[k++] = argv[j] ;
      else          { usage () ; return -1 ; }

	if ( k != 4 )
    { usage () ;  return -1 ; }

  return 0 ;
}

int main ( INT argc, char *argv[] )
{
  struct_param param ;
  FILE *infn, *infc, *infm, *infd ;

  fputs("Signal mixer. Ver.1.1 by JK & PP at CTU FEL K331. 5 Nov 97\n",stderr) ;

  /*  Initialization */
  param.wlen = 256 ;
  param.wstep = 128 ;
  param.lengthc = MAXLONG ;
  param.snr = 0 ;
  param.extwin=0 ;

  if ( 0 != param_set ( argc, argv, &param ) )
     return -1 ;

  if ((infc=fopen(param.filename[0],"rb"))==NULL)
   { fprintf(stderr,"mix: Cannot open %s for reading.\n",param.filename[0]) ;
     return -1 ; }

  if ((infn=fopen(param.filename[1],"rb"))==NULL)
   { fprintf(stderr,"mix: Cannot open %s for reading.\n",param.filename[1]) ;
     return -1 ; }

  if ((infm=fopen(param.filename[2],"wb"))==NULL)
   { fprintf(stderr,"mix: Cannot open %s for writing.\n",param.filename[2]) ;
     return -1 ; }

  if ((infd=fopen(param.filename[3],"rt"))==NULL)
   { fprintf(stderr,"mix: Cannot open %s for reading.\n",param.filename[3]) ;
     return -1 ; }

  snr_comp ( infc, infn, infm, infd,
        param.snr, param.wlen, param.wstep, param.lengthc,param.extwin ) ;

  fclose(infn) ; fclose(infc) ; fclose(infm) ; fclose(infd) ;

  return 0 ;
}
