/* This is a header file for 'C' implemetations */
/* Noisy speech enhancement algorithm. 
   Two step spectral substraction, full-wave rectification */
/* Jan Kybic */

/* First we set the global parameters */
#ifndef SO1FF_H
#define SO1FF_H

/* readftype is a type for function to be called from within so1ff_do
   to get a chunk of input data. The function should store them as floats
   into an area pointed to by p. It should return 1 if there is no more
   data available, negative number on errors and 0 otherwise. */
    
#ifndef READFTYPEDEF
#define READFTYPEDEF    
typedef int readftype(float *p) ;
#endif

/* so1ff_init initializes the module. f should point to the function to
   be called to obtain data. so1ff_init sets *sizep to the size of chunks
   it wishes to communicate in, usually a power of 2 nad *outbuf to the address
   of an output buffer. Caller must not change it! 
   It returns 0 if no error occured. det is a pointer to a command line
   argument that is used to determine the type of detector to use. */

struct so1ff_info { char *detfile ;
                    char twice ;
                    char fullwave ; } ;
                    
                    

int so1ff_init(readftype *f,int *sizep,float **outbuf,void *info) ;

int so1ff_close() ; /* clean up */

/* so1ff_do attempts to perform the desired noise reduction and stores
   the next *sizep output samples into an area pointed to by outbuf.
   Return values are the same as for the readftype *f function */
    
int so1ff_do() ;

#endif