/* This is the main source file for program ssub, 
   performing spectral subtraction on 16-bit, 8kHz, mono raw sound files.
   
   Jan Kybic, February 1996
*/

#include <stdio.h>
#include "martin.h"
#include "dobl.h"
#include "so1ff.h"
 
#define MAX_CHUNK 512 
#define INT short int
#define MAXINT 32767

static FILE *inf,*outf ;
static int chunk_size ;
static float *outbuf,*fp ;
static INT ibuf[MAX_CHUNK],*ip ; /* integer input/output buffer */

/* Pointers to executive functions */
static int (*xx_init)(readftype *f,int *sizep,float **outbuf,char *det) ;
static int (*xx_close)() ;
static int (*xx_do)() ;

/* call back function */
static int getf(float *p)
{ int r ; 

  r=fread(ibuf,sizeof(INT),chunk_size,inf) ;
  if (r!=chunk_size) return(1) ; 
      
  for(ip=ibuf,fp=p;ip<ibuf+chunk_size;ip++,fp++) *fp=*ip ;
  return(0) ;
} 
 
 
struct so1ff_info info ; 
 
void main(int argc,char *argv[])
{ int r ; 
  int ofs ; /* parameter offset */

  puts("ssub version 1.1, using algorithm by Rainer Martin, Gerhard Doblinger") ;
  puts("                                    or classical spectral subtraction") ; 
  if (argc<2)
   { puts("usage: ssub [options] input_file.bin output_file.bin") ;
     puts("options: -m       use algorithm by Martin") ;
     puts("         -o       use algorithm by Doblinger") ;
     puts("         -d<file> read detector output from file") ;
     puts("         -d       no detector") ;
     puts("         -h       half wave rectification") ;
     puts("         -1       subtract only once") ;
     puts("The default is to use spectral subtraction algorithm with built-in") ;
     puts("energy detector, full wave rectification applied twice.") ;
     puts("We assume 16bit, signed, mono, 8000Hz signal with no header.") ;
     return ;
   }
   
  xx_init=(void *)so1ff_init ;
  xx_close=so1ff_close ;
  xx_do=so1ff_do ;
  info.detfile=NULL ; info.twice=1 ; info.fullwave=1 ;
   
  for (ofs=1;ofs<argc;ofs++)
    { if (argv[ofs][0]!='-') break ;
      switch(argv[ofs][1])
       { case 'm': xx_init=(void *)martin_init ;
                   xx_close=martin_close ;
                   xx_do=martin_do ;
                   break ;
         case 'o': xx_init=(void *)dobl_init ;
                   xx_close=dobl_close ;
                   xx_do=dobl_do ;
                   break ;
                   
         case 'd': info.detfile=argv[ofs]+2 ;
                   break ;
         case '1': info.twice=0 ;
         	   break ;
         case 'h': info.fullwave=0 ;
                   break ;	   
         default:  puts("Invalid option. For help, invoke ssub without arguments.") ;          
                   return ;
       }
    }  

  if (ofs+1>=argc)
    { puts("ssub: Not enough arguments") ;
      return ; 
    }  
   
  if ((inf=fopen(argv[ofs],"r"))==NULL)
   { printf("ssub: !!! Cannot open %s for reading.\n",argv[ofs]) ;
     return ; }

  if ((outf=fopen(argv[ofs+1],"w"))==NULL)
   { printf("ssub: !!! Cannot open %s for writing.\n",argv[ofs+1]) ;
     fclose(inf) ; return ; }
         
         
  if (xx_init(getf,&chunk_size,&outbuf,(void *)&info)) 
        { puts("ssub: !!! xx_init failed.") ;
          fclose(inf) ; fclose(outf) ;
          return ; }
     
  if (chunk_size>MAX_CHUNK)
    { puts("ssub: !!! xx_init request too high chunk_size.") ;
      fclose(inf) ; fclose(outf) ;
      return ; }
         
     
  while ((r=xx_do())==0) 
    { 
      for(ip=ibuf,fp=outbuf;ip<ibuf+chunk_size;ip++,fp++) 
        if (*fp>(float)MAXINT) *ip=MAXINT ;
          else if (*fp<-(float)MAXINT) *ip=-MAXINT ;
                 else *ip=(INT) *fp ;
      fwrite(ibuf,sizeof(INT),chunk_size,outf) ;
    }
  if (r!=1)
     puts("ssub: !!! xx_do failed.") ;
  
  fclose(inf) ; fclose(outf) ;    
}     
