/*
  tstlsw    The (reduced) MEX version of tstlsw

% Toeplitz Structured Total Least Squares Algorithm
% 'x' containts measurements corrupted by white noise
% on output 'a' contains the coefficients of an AR system of the order 'n'
% generating output 'y', while (x-y) is minimised in Euclidean sense
% y(i)=a(1)*y(i-1)+a(2)*y(i-2)+...+a(n)*y(i-n)
% a=arg min sum((y(i)-x(i))^2)
%
% Uses weighting to get unconstrained problem that solves using (conjugate) 
% gradient method
%
% Jan Kybic, 1997
%
% Usage: a=tstlswc(x,n,[maxiter])

*/


#ifdef __GNUC__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#define myCreateMatrix(a,b) mxCreateFull(a,b,REAL)
#define myAlloc(a) mxCalloc(a,1)
#else /* hp version */
#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#endif

#include <string.h>
#include "mex.h"

#define MAXITER (20)   /* default maximum number of iterations */
int maxiter ;

/* Input Arguments */

#define	X_IN	prhs[0]
#define	N_IN	prhs[1]
#define MAXITER_IN prhs[2]

/* Output Arguments */

#define	A_OUT	plhs[0]

void tstlsw( double *x, double *a, int n, int T,int maxiter) ;
 
#ifdef linux
void mexFunction(
                 int nlhs, Matrix *plhs[],
                 int nrhs, Matrix *prhs[]
		 )
#else
void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
#endif
{ int T,n ;

  double *ap,*xp,*np,*maxiterp ;

  /* Check for proper number of arguments */
  
  if (nrhs != 2 && nrhs!=3) {
    mexErrMsgTxt("TSTLSWC requires two or three input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt("TSTLSWC requires one output argument.");
  }
  
  if (!mxIsNumeric(X_IN) || mxIsComplex(X_IN) || 
      mxIsSparse(X_IN)  || !mxIsDouble(X_IN)) 
         mexErrMsgTxt(
             "TSTLSWC needs the first input argument to be a real vector.");
      
  if (!mxIsNumeric(N_IN) || mxIsComplex(N_IN) || 
      mxIsSparse(N_IN)  || !mxIsDouble(N_IN)) 
         mexErrMsgTxt(
             "TSTLSWC needs the second input argument to be a scalar.");

  if (nrhs==3)
  if (!mxIsNumeric(MAXITER_IN) || mxIsComplex(MAXITER_IN) || 
      mxIsSparse(MAXITER_IN)  || !mxIsDouble(MAXITER_IN)) 
         mexErrMsgTxt(
             "TSTLSWC needs the third input argument to be a scalar.");
      
  if (mxGetM(X_IN)==1) T=mxGetN(X_IN) ; else
  if (mxGetN(X_IN)==1) T=mxGetM(X_IN) ; else
         mexErrMsgTxt(
             "TSTLSWC needs the first input argument to be a real vector.");
 
  if (mxGetM(N_IN)!=1 || mxGetN(N_IN)!=1)  
         mexErrMsgTxt(
             "TSTLSWC needs the second input argument to be a scalar.");
  if (nrhs==3)
  if (mxGetM(MAXITER_IN)!=1 || mxGetN(MAXITER_IN)!=1)  
         mexErrMsgTxt(
             "TSTLSWC needs the third input argument to be a scalar.");

  xp=mxGetPr(X_IN) ;
  np=mxGetPr(N_IN) ;
  maxiterp=mxGetPr(MAXITER_IN) ;
  if (*np!=(double)(n=(int)(*np)) || n<1)
     mexErrMsgTxt(
      "TSTLSWC needs the second input argument to be a positive integer.");

  if (nrhs==3) {
    if (*maxiterp!=(double)(maxiter=(int)(*maxiterp)) || maxiter<1)
      mexErrMsgTxt("TSTLSWC needs the third input argument to be a positive integer.");
  } else maxiter=MAXITER ;

  A_OUT = myCreateMatrix(1,n);

  ap = mxGetPr(A_OUT); 

  /*  %%%%% here the calculation begins %%%%%% */
  tstlsw(xp,ap,n,T,maxiter) ;

  return;
}

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt("TSTLSWC: could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt("TSTLSWC: could not malloc") ;

#define MAKING_MEX
#include "../c/tstlsw1.c"
