function [y, te]=arkalm(z,as,bs,r) ;
% This function takes the corrupted signal z=signal+noise,
% AR parameters of signal (as,bs) and noise covariance.
% Noise is assumed to be white, Gaussian, with covariance r.
% Estimate y of the signal using bidirectional nonstationary Kalman filter.
% te is the total forward prediction error
% Usage: [y,te]=arkalm(z,as,bs,r) ;

A=ar2tm(as) ;
n=length(as) ;
T=length(z) ;
C=[zeros(1,n-1) 1] ;
Q=zeros(n) ; Q(n,n)=bs^2 ;
% P=c2R(cs(1:n)) ;
P=lyap1(A,Q) ;

% initial uninformative guess
x=zeros(n,1) ;
xf=zeros(n,T) ;
PfA=zeros(n,n) ;
xp=zeros(n,T) ;
F=zeros(n,n) ;
Fp=zeros(T,n*n) ;
y=zeros(1,n) ;

te=0 ;
% Forward run
for i=1:T,
 % Data step
 e=z(i)-x(n) ;          % prediction error
 te=te+e^2 ;
L=P*C' / (C*P*C'+r) ;
% L=P(:,n) / (P(n,n)+r) ; % Kalman gain

x=x+L*e ;             % correct the state estimate

P=P-L*C*P ;
% P=P-L*P(n,:) ;

 xf(:,i)=x ;              % filtered estimate
 
 % Time step
 x=A*x ;
 PfA=P*A' ;
 P=A*PfA+Q ;

 xp(:,i)=x ;              % predicted estimate
 F=PfA/P ;
 Fp(i,:)=reshape(F,1,n*n) ;
end ;

% Backward run
xl=x ;
for i=T:-1:1,
  F=reshape(Fp(i,:),n,n) ;
  x=xf(:,i)+F*(xl-xp(:,i)) ;
  xl=x ;
  y(i)=x(n) ;
end ;


