function [ys,k,inov,err]=arxeokal(y,a,q,r)
% Perform Kalman filter for ARX model and known parameters a
% q is the system noise covariance and r the output noise covariance
% k is the steady-state Kalman gain row vector k1,...,kn 
% Usage: [ys k]=arxoekal(y,a,q,r)

n=length(a) ; T=length(y) ;
ys=zeros(1,T) ;
ys(1:n)=y(1:n) ;
inov=zeros(1,T) ;

A=zeros(n) ;
A(1,1:n)=a ;
for i=2:n, A(i,i-1)=1 ; end ;

P=5*eye(n)*r ;

x=y(1)*ones(n,1) ;
for i=1:n, x(n+1-i)=y(i) ; end ;

% the first time step
x=A*x ;

err=0 ;
for i=n+1:T,
   % store the predicted estimate
   ys(i)=x(1) ;
   % make the data step
   inov(i)=y(i)-x(1) ;
   err=err+inov(i)*inov(i) ;
   x=x+P(:,1)/(P(1,1)+r)*inov(i) ;

   % perform the time step
   x=A*x ;
   P=A*(P-P(:,1)*P(1,:)/(P(1,1)+r))*A' ;
   P(1,1)=P(1,1)+q ;
   end ;
end ;
k=(P(:,1)/(P(1,1)+r))' ;


