function  [a,alfa,rc]=burg(x,p);

% Function BURG: Computation of AR model using Burg's algorithm.
%
% Usage: [a,alpha,rc]=burg(x,p);
% Input parameters:
%   x    - processing frame
%   p    - order of LPC model
% Output parameters:
%   a     - autoregressive coefficients 
%   alpha - forward prediction error
%   rc    - reflection coefficients
%

%                            Made by PP
%                            4 Apr 1995
%                       CVUT FEL K331, Prague
%%
%  changed by JK, 11.9.1997 to avoid division by zero 
%  changed output format (skipped first coefficient and changed sign)

x=x(:);
delka=length(x);

ef = x;
eb = x;

alfa = sum(ef.*ef)/delka;

  ik=1;  
  den = sum(ef(1+ik:delka).*ef(1+ik:delka)) + sum(eb(ik:delka-1).*eb(ik:delka-1));
  num = 2*sum(ef(1+ik:delka).*eb(ik:delka-1));
% here we might have divided by zero
  if den>eps,
    rc(ik) =  - num / den;
    a(ik) = - num / den ;
  else  
    rc(ik)=0 ; a(ik)=0 ;
  end ;
  efold = ef;
  ebold = eb;
  ef(2:delka) = ef(2:delka) + rc(ik)*ebold(1:delka-1);
  eb(2:delka) = ebold(1:delka-1) + rc(ik)*efold(2:delka);
  alfa = alfa*(1-rc(ik)*rc(ik));
  aa = a;

for ik=2:p,  
  den = sum(ef(1+ik:delka).*ef(1+ik:delka)) + sum(eb(ik:delka-1).*eb(ik:delka-1));
  num = 2*sum(ef(1+ik:delka).*eb(ik:delka-1));
  if den>eps,  
    rc(ik) =  - num / den;
    a(ik) = - num / den ;
  else  
    rc(ik)=0 ; a(ik)=0 ;
  end ;
  efold = ef;
  ebold = eb;
  ef(2:delka) = ef(2:delka) + rc(ik)*ebold(1:delka-1);
  eb(2:delka) = ebold(1:delka-1) + rc(ik)*efold(2:delka);
  alfa = alfa*(1-rc(ik)*rc(ik));
  a(1:ik-1) = aa(1:ik-1) + rc(ik)*fliplr(aa(1:ik-1));
  aa = a;
end;

a=-a;




