function a=c2ach(c,n) 
% Given an autocorrelation function c, this function computes the 'a'
% parameters of an AR synthetising filter of the order n
% It uses normal equations derived from Cayley-Hamilton theorem
% c should begin with autocorrelation at lag 0
% Usage: a=c2ach(c,n)

c=c(:) ;
% The length of the input vector
T=length(c) ;
if (2*n>=T), error('The signal is too short.') ; end ;

c=c(2:2*n+1) ; % shift

% Arrange the coefficients into a matrix
C=zeros(n) ;
for i=1:n, C(:,i)=c(i:i+n-1) ; end ;

% Compute the AR model parameters
a=(C\c(n+1:2*n))' ;
 
% Revert the a-coefficients into the usual order
a=a(n:-1:1) ;




