function a=c2ache(c,n) 
% Given an autocorrelation function c, this function computes the 'a'
% parameters of an AR synthetising filter of the order n
% It uses normal equations derived from Cayley-Hamilton theorem
% c should begin with autocorrelation at lag 0
% It differs from c2ach in that it uses the c vector in its entirety
% Usage: a=c2ache(c,n)


c=c(:) ;
% The length of the input vector
T=length(c) ;
if (2*n>=T), error('The signal is too short.') ; end ;

mh=T-n-1 ; % number of equations

c=c(2:T) ; % shift

% Arrange the coefficients into a matrix
C=zeros(mh,n) ;
for i=1:n, C(:,i)=c(i:i+mh-1) ; end ;

% Compute the ARX model parameters
a=(C\c(n+1:mh+n))' ;

% Revert the a-coefficients into the usual order
a=a(n:-1:1) ;









