function a=c2achw(c,w,n) 
% Given an autocorrelation function c, this function computes the 'a'
% parameters of an AR synthetising filter of the order n
% It uses normal equations derived from Cayley-Hamilton theorem
% c should begin with autocorrelation at lag 0
% w are the weights associated with corresponding member of c
% It differs from c2ache in that it uses w to remove the bias
% Usage: a=c2ache(c,w,n)


c=c(:) ;
w=w(:) ;
% The length of the input vector
T=length(c) ;
if (2*n>=T), error('The signal is too short.') ; end ;
if (length(w)~=T), error('w and c should have the same length.') ; end ;

mh=T-n-1 ; % number of equations

c=c(2:T) ; % shift
w=w(2:T) ;

% Arrange the coefficients into a matrix
C=zeros(mh,n) ;
for i=1:mh, 
  C(i,:)=(c(i:i+n-1) ./ (w(i:i+n-1) / w(i+n)))'  ; 
end ;

% Compute the ARX model parameters
a=(C\c(n+1:mh+n))' ;

% Revert the a-coefficients into the usual order
a=a(n:-1:1) ;









