function aout=clhamred(x,n) 
% Given a signal x, this function uses Cayley-Hamilton theorem to estimate the
% a parameters of an ARX model of the order n that gives the same output x 
% when fed with white noise. x must be at least 2n elements long.
% This routine uses more than 2n autocorr. coefs and does repetitive
% autocorrelation to improve efficiency.
% Usage: a=clhamre(x,n)

% The length of the input vector
T=length(x) ;
m=max(2*n,1000) ;  % We shall use this many coefficients
m=min(m,T) ;

if (2*n>T), error('The signal is too short.') ; end ;

w=ones(m,1) ; %initial weights
xc=x ;

mh=T-n ;
 C=zeros(mh,n) ;
 for i=1:mh, 
     C(i,:)=( xc(i:i+n-1)' ./ (w(i:i+n-1) / w(i+n)) )' ; 
     % C(i,:)=xc(i:i+n-1)' ;
 end ;
 a=(C\xc(n+1:T)')' ;
 aout(1,1:n)=a(n:-1:1) ;
 aout(1,n+1)=mlaroefn2(aout(1,1:n),x,0) ;

iter=15 ; % Number of iterations
for j=1:iter,
 xc=corrn(xc,m+1-j) ; % xc(1)
 xc=xc(2:m+1-j) ; xc=xc/max(abs(xc)) ; % scaling seems necessary
 w=corrn(w,m+1-j) ;
 w=w(2:m+1-j) ; w=w/max(w) ; 
% plot([xc./w w]) ; axis([0 1000 -2 2]) ;
if 1, % compute on the way
 C=zeros(n) ;
 for i=1:n, 
     C(i,:)=( xc(i:i+n-1) ./ (w(i:i+n-1) / w(i+n)) )' ; 
     % C(i,:)=xc(i:i+n-1)' ;
 end ;
 a=(C\xc(n+1:2*n))' ;
 aout(j+1,1:n)=a(n:-1:1) ;
 aout(j+1,n+1)=mlaroefn2(aout(j+1,1:n),x,0) ;

% 1 ./ abs(log(roots([1 -a])))'
% pause
end ;

 end ; % of for j

% Arrange the coefficients into a Toeplitz matrix
mh=length(xc)-n ; % the number of equations
%mh=n ;
C=zeros(mh,n) ;
for i=1:mh, 
     C(i,:)=( xc(i:i+n-1) ./ (w(i:i+n-1) / w(i+n)) )' ; 
end ;

% Compute the ARX model parameters
a=(C\xc(n+1:mh+n))' ;
 
% Revert the a-coefficients into the usual order
aout(iter+2,1:n)=a(n:-1:1) ;
aout(iter+2,n+1)=mlaroefn2(aout(iter+2,1:n),x,0) ;









