function a=clhamres(x,n) 
% Given a signal x, this function uses Cayley-Hamilton theorem to estimate the
% a parameters of an ARX model of the order n that gives the same output x 
% when fed with white noise. x must be at least 2n elements long.
% This routine uses more than 2n autocorr. coefs and does repetitive
% autocorrelation to improve efficiency.
% Spectral estimation of the autocorrelatoin is used
% Usage: a=clhamres(x,n)

% The length of the input vector
T=length(x) ;
m=max(2*n,1000) ;  % We shall use this many coefficients
m=min(m,T) ;

if (2*n>T), error('The signal is too short.') ; end ;

xc=x ;

iter=7 ; % Number of iterations
for j=1:iter,
 xc=corrs(xc,m-j+1)' ; % xc(1)
 xc=xc(2:m-j+1) ; xc=xc/max(abs(xc)) ; % scaling seems necessary

if 0, % compute on the way
  mh=length(xc)-n ; % the number of equations
  C=zeros(mh,n) ;
  for i=1:mh, C(i,:)=xc(i:i+n-1) ; end ;

  % Compute the ARX model parameters
  a=(C\xc(n+1:mh+n)')' ;
 
  % Revert the a-coefficients into the usual order
  a=a(n:-1:1) 

end ; % end of if

 end ; % of for j

% Arrange the coefficients into a matrix
mh=length(xc)-n ; % the number of equations
C=zeros(mh,n) ;
for i=1:mh, 
     C(i,:)=xc(i:i+n-1) ; 
end ;

% Compute the ARX model parameters
a=(C\xc(n+1:mh+n)')' ;
 
% Revert the a-coefficients into the usual order
a=a(n:-1:1) ;









