function y=darkalm2(z,as,bs,cs,an,bn,cn) ;
% This function takes the corrupted signal z=signal+noise,
% AR parameters of signal (as,bs) and noise (an,bn)
% and signal resp. noise autocorrelations (cs,cn) and produces the smoothed
% estimate y of signal using bidirectional nonstationary Kalman filter
% It uses the modified smoothing formulas and is therefore quicker than darkalm
% it is optimized for AR models but not as much as darklam3

[A,C,Q,r,P]=ard2tme(as,bs,cs,an,bn,cn) ;

% r=(cn(1)+cs(1))*1e-6 ;

ns=length(as) ;
nn=length(an) ;
n=length(A) ;
T=length(z) ;

% initial uninformative guess
x=zeros(n,1) ;
xf=zeros(n,T) ;
Pp=zeros(T,n*n) ;
Lk=zeros(T,n) ;
pomp=zeros(T) ;
y=zeros(1,n) ;
ek=zeros(T) ;

% Forward run
for i=1:T,
 % Data step
 % e=z(i)-C*x ;  
 e=z(i)-(x(nn)+x(n)) ;          % prediction error
 ek(i)=e ;
 
 % K=C' / (C*P*C'+r) ;
 % L=P*K ;
 pom=1/(P(n,n)+P(ns,ns)+2*P(ns,n)+r) ;
 pomp(i)=pom ;
 K=C'*pom ;
 L=(P(:,n)+P(:,nn)) * pom  ; % Kalman gain
 Lk(i,:)=L' ;
 x=x+L*e ;             % correct the state estimate

 % P=P-L*C*P ;
 P=P-L*(P(n,:)+P(nn,:)) ;

 xf(:,i)=x ;              % filtered estimate
 Pp(i,:)=reshape(P,1,n*n) ;
 
 % Time step
 x=A*x ;
 P=A*P*A'+Q ;
end ;

% Backward run
lmb=zeros(n,1) ;
for i=T:-1:1,
  K=C'*pom ;
  L=Lk(i,:)' ;
  P=reshape(Pp(i,:),n,n) ;
  Almb=A'*lmb ;
  PAlmb=P*Almb ;
  x=xf(:,i)+PAlmb ;
  y(i)=x(n) ;
% y(i)=xf(n,i) ; % this would use only the forward run

%  lmb=Almb+(C'/r*C*Lk(i,:)'+K)*((1-C*Lk(i,:)')*ek(i)-C*PAlmb) ;
%  lmb=Almb+(C'*(L(n)+L(nn))/r+K)*((1-L(n)-L(nn))*ek(i)-PAlmb(n)-PAlmb(nn)) ;

  Ls=L(n)+L(nn) ; Lr=Ls/r ;
  KLr=K ; KLr(n)=KLr(n)+Lr ; KLr(nn)=KLr(nn)+Lr ; 
  lmb=Almb+KLr*((1-Ls)*ek(i)-PAlmb(n)-PAlmb(nn)) ;
end ;

% Here we check for overflows
sy=sum(abs(y))/T ;
if isnan(sy) | sy>10*max(abs(z)),
  y=xf(n,:) ;
end ;



