function y=darkalm3(z,as,bs,cs,an,bn,cn) ;
% This function takes the corrupted signal z=signal+noise,
% AR parameters of signal (as,bs) and noise (an,bn)
% and signal resp. noise autocorrelations (cs,cn) and produces the smoothed
% estimate y of signal using bidirectional nonstationary Kalman filter
% Usage: y=darkalm3(z,as,bs,cs,an,bn,cn) ;

% it is optimized for AR models but not with respect to Matlab
% but rather with respect to the number of operations

ns=length(as) ;
nn=length(an) ;
n=ns+nn ;

A=zeros(n) ; % system transfer matrix
P=zeros(n) ; % initial state estimate covariance matrix

C=[zeros(1,nn-1) 1 zeros(1,ns-1) 1] ; % output matrix (vector, actually)

A(1:nn,1:nn)=ar2tm(an) ;
A(nn+1:n,nn+1:n)=ar2tm(as) ;

qn=bn^2 ;
qs=bs^2 ;

r=(qn+qs)*1e-6 ; % rather heuristic setting

P(1:nn,1:nn)=c2R(cn(1:nn)) ;
P(nn+1:n,nn+1:n)=c2R(cs(1:ns)) ;

ns=length(as) ;
nn=length(an) ;
n=length(A) ;
T=length(z) ;

% initial uninformative guess
x=zeros(n,1) ;
xf=zeros(n,T) ;
Pp=zeros(T,n*n) ;
Lk=zeros(T,n) ;
pomp=zeros(T) ;
y=zeros(1,n) ;
ek=zeros(T) ;
Almb=zeros(n,1) ;

% Forward run
for i=1:T,
 % Data step
 % e=z(i)-C*x ;  
 e=z(i)-(x(nn)+x(n)) ;          % prediction error
 ek(i)=e ;
 
 % K=C' / (C*P*C'+r) ;
 % L=P*K ;
 pom=1/(P(n,n)+P(ns,ns)+2*P(ns,n)+r) ;
 pomp(i)=pom ;
 K=C'*pom ;
 L=(P(:,n)+P(:,nn)) * pom  ; % Kalman gain
 Lk(i,:)=L' ;
 x=x+L*e ;             % correct the state estimate

 % P=P-L*C*P ;
 P=P-L*(P(n,:)+P(nn,:)) ;

 xf(:,i)=x ;              % filtered estimate
 Pp(i,:)=reshape(P,1,n*n) ;

 % Time step
 % x=A*x ;
 xnew=as(ns:-1:1)*x(nn+1:n) ;
 x(nn+1:n-1)=x(nn+2:n) ; 
 x(n)=xnew ;
 xnew=an(nn:-1:1)*x(1:nn) ;
 x(1:nn-1)=x(2:nn) ; 
 x(nn)=xnew ;

% P=A*P*A'+Q ;
% P=A*P*A' ;
 for ii=1:n,
   xnew=as(ns:-1:1)*P(nn+1:n,ii) ;
   P(nn+1:n-1,ii)=P(nn+2:n,ii) ; 
   P(n,ii)=xnew ;
   xnew=an(nn:-1:1)*P(1:nn,ii) ;
   P(1:nn-1,ii)=P(2:nn,ii) ; 
   P(nn,ii)=xnew ;
 end ;
 for ii=1:n,
   xnew=as(ns:-1:1)*P(ii,nn+1:n)' ;
   P(ii,nn+1:n-1)=P(ii,nn+2:n) ; 
   P(ii,n)=xnew ;
   xnew=an(nn:-1:1)*P(ii,1:nn)' ;
   P(ii,1:nn-1)=P(ii,2:nn) ; 
   P(ii,nn)=xnew ;
 end ;

 P(n,n)=P(n,n)+qs ;
 P(nn,nn)=P(nn,nn)+qn ;
end ;

% Backward run
lmb=zeros(n,1) ; y(T)=xf(n,T) ;
for i=T:-1:1,
  L=Lk(i,:)' ;
  P=reshape(Pp(i,:),n,n) ;
  % Almb=A'*lmb ;
  Almb(1)=lmb(nn)*an(nn) ;
  Almb(2:nn)=lmb(1:nn-1)+lmb(nn).*an(nn-1:-1:1)' ;
  Almb(nn+1)=lmb(n)*as(ns) ;
  Almb(nn+2:n)=lmb(nn+1:n-1)+lmb(n).*as(ns-1:-1:1)' ;
  PAlmb=P*Almb ;
  x=xf(:,i)+PAlmb ;
  y(i)=x(n) ;
  Ls=L(n)+L(nn) ; Lr=Ls/r ;
  lmb=Almb ; g=(pomp(i)+Lr)*((1-Ls)*ek(i)-PAlmb(n)-PAlmb(nn)) ;
  lmb(n)=lmb(n)+g ; lmb(nn)=lmb(nn)+g ;
end ;





