function [y, te]=darkalmsb(z,as,bs,cs,an,bn,cn) ;
% This function takes the corrupted signal z=signal+noise,
% AR parameters of signal (as,bs) and noise (an,bn)
% and signal resp. noise autocorrelations (cs,cn) and produces the smoothed
% estimate y of signal using bidirectional nonstationary Kalman filter
% te is the total forward prediction error
% it is optimized for AR models but not with respect to Matlab
% but rather with respect to the number of operations
%
% Covariance Square Root Filter
%
% Usage [y, te]=darkalmsb(z,as,bs,cs,an,bn,cn)

[A,C,Q,r,P]=ard2tme(as,bs,cs,an,bn,cn) ;

ns=length(as) ;
nn=length(an) ;
n=length(A) ;
T=length(z) ;

% initial uninformative guess
x=zeros(n,1) ;
y=zeros(1,n) ;
xf=zeros(n,T) ;
PfA=zeros(n,n) ;
xp=zeros(n,T) ;
F=zeros(n,n) ;
Fp=zeros(T,n*n) ;
te=0 ;
P=dlyap2(A,Q) ;
M=chol(P) ; % M is now upper triangular
sr=sqrt(r) ;
Qs=diag(sqrt(diag(Q))) ;

% Forward run
for i=1:T, 
% sum(diag(P*P'))
 % Data step
 e=z(i)-(x(nn)+x(n)) ;          % prediction error
 te=te+e^2 ;
 Z=[sr zeros(1,n) ; M*C' M ] ;
% [zq Z]=qr(Z) ;
 Z=chol(Z'*Z) ;
 L=Z(1,2:n+1)'/Z(1,1) ;
 M=Z(2:n+1,2:n+1) ;

% L=(P(:,n)+P(:,nn)) / (P(n,n)+P(ns,ns)+2*P(ns,n)+r) ; % Kalman gain

 x=x+L*e ;             % correct the state estimate
 xf(:,i)=x ;  
   y(i)=x(n) ;
% P=P-L*C*P ;
%  P=P-L*pom*L' ;
%  IL=eye(n)-L*C ;
%  P=IL*P ;
%  P=IL*P*IL'+L*r*L' ;
% P=P-L*(P(n,:)+P(nn,:)) ;

 % Time step
 x=A*x ;
 xp(:,i)=x ;              % predicted estimate

 Z=[M*A' ; Qs] ;
 %  [zq Z]=qr(Z) ;
 Z=chol(Z'*Z) ;
 PfA=M'*M*A' ;
 M=Z(1:n,:) ;
% F=(PfA/M)/M' ;
 F=PfA/(M'*M) ;
 Fp(i,:)=reshape(F,1,n*n) ;
end ;

% Backward run
xl=x ;
for i=T:-1:1,
  F=reshape(Fp(i,:),n,n) ;
  x=xf(:,i)+F*(xl-xp(:,i)) ;
  xl=x ;
  y(i)=x(n) ;
end ;



