function [y, te]=darkalmu(z,as,bs,cs,an,bn,cn) ;
% This function takes the corrupted signal z=signal+noise,
% AR parameters of signal (as,bs) and noise (an,bn)
% and signal resp. noise autocorrelations (cs,cn) and produces the smoothed
% estimate y of signal using unidirectional nonstationary Kalman filter
% te is the total forward prediction error
% it is optimized for AR models but not with respect to Matlab
% but rather with respect to the number of operations
%
% Covariance Square Root Filter
%
% Usage [y, te]=darkalmu(z,as,bs,cs,an,bn,cn)

[A,C,Q,r,P]=ard2tme(as,bs,cs,an,bn,cn) ;

ns=length(as) ;
nn=length(an) ;
n=length(A) ;
T=length(z) ;

% initial uninformative guess
x=zeros(n,1) ;
y=zeros(1,n) ;

te=0 ;
% Forward run
for i=1:T, 
% sum(diag(P*P'))
 % Data step
 e=z(i)-(x(nn)+x(n)) ;          % prediction error
 te=te+e^2 ;
 pom=1/(C*P*C'+r) ;
 L=P*C'*pom  ;
% L=(P(:,n)+P(:,nn)) / (P(n,n)+P(ns,ns)+2*P(ns,n)+r) ; % Kalman gain

x=x+L*e ;             % correct the state estimate

% P=P-L*C*P ;
%  P=P-L*pom*L' ;
  IL=eye(n)-L*C ;
  P=IL*P ;
%  P=IL*P*IL'+L*r*L' ;
% P=P-L*(P(n,:)+P(nn,:)) ;

 y(i)=x(n) ; 
 
 % Time step
 x=A*x ;
% xnew=A(n,nn+1:n)*x(nn+1:n) ;
% x(nn+1:n-1)=x(nn+2:n) ; 
% x(n)=xnew ;
% xnew=A(nn,1:nn)*x(1:nn) ;
% x(1:nn-1)=x(2:nn) ; 
% x(nn)=xnew ;

 PfA=P*A' ;
 P=A*PfA+Q ;
end ;




