function [y,e]=fltseg(x) ;
% Filter segment once
% Takes a segment x of the noisy input signal, estimates parameters
% of its AR model and the noise covariances and applies Kalman
% filter to obtain the filtered signal y and innovation e
% Usage: [y e]=fltseg(x) ;

n=4 ; % predetermined order of the system
T=length(x) ;

% Estimate ARX parameters through autocorrelation
a=x2ar(x,n) ;
a=astab(a) ;
A=ar2tm(a) ; % system matrix

% Estimate the noise covariances
m=min(T,200) ; % number of coefficients
c=corrs(x,m) ; % calculate coefficients
[q r]=getnoise(c,a) ; % process noise, measurement noise

% Calculate Kalman gain
k=kgain(A,q/r) ;

% Filter
[y e]=kalmk(x,A,k) ;





