function y=fltsnd(x) ;
% Filters a sound signal 
% based on Kalman filtration
% Usage: y=fltsnd(x), where x is the signal to be filtered (row vector)
% Jan Kybic, 14.3.1997
% changed 22.4.1997
% changed 1.6.1997

w=256 ; 	    % window length
n=4 ; 		    % order of the system	
kini=0.1*ones(1,n) ;   % initial Kalman gain
k=kini ;
T=length(x) ;
hann=hanning(w)' ;
y=zeros(1,T) ;      % output vector

x=x-sum(x)/T ;
dpx=sum(x.*x) ;

% We use w/2 overlap
lastl=floor(2*T/w)-2 ;
for l=0:lastl, % l is the window index
  disp(['Processing window ',num2str(l)]) ; 
  begi=l*(w/2)+1 ; endi=l*(w/2)+w ;
  xw=x(begi:endi) ; % get one window

  a=clhamre(xw,n) ; % find the AR model parameters
  A=ar2tm(a) ;
  % Fix for unstable matrixes
  while (max(abs(roots(poly(A))))>0.999), A=0.99*A ; end ;

  % Filter, adaptively find the correct Kalman gain
  % Unfortunately, we may not use the old values
  [xw e k]=kalmkee(xw,A,kini) ;
  % here we take care about the windowing with proper
  %       beginning/end
  xwh=xw .* hann ;
  if l==1,         xwh=[xw(1:w/2) xwh(w/2+1:w)] ; 
  elseif l==lastl, xwh=[xwh(1:w/2) xw(w/2+1:w)] ;
  end ;
  y(begi:endi)=y(begi:endi)+xwh ;
end ;

% Adjust the mean to zero 
y=y-sum(y)/T ;
     	





