function sn=getpns(x)
% This function estimates the power spectrum of the noise signal
% using minima tracking algorithm described by Doblinger, EUROSPEECH'95
% Usage: sn=getpns(x)

% Set some constants first
w=256   ; % window length, should be power of two
ovrf=2  ; % overlap factor

% Algorithm parameters - original
% alpha=0.7 ;
% beta=0.96 ;
% gamma=0.998 ;
alpha=0.9 ;
beta=0.9 ;
gamma=0.998 ;

s=w/ovrf ; % window shift

x=x(:)' ;
T=length(x) ;

cnt=floor((T-w)/s) ; % how many windows
h=hanning1(w) ;       % window weights
h=w/ovrf/sum(h)*h ;  % adjust to proper scaling


x=x-sum(x)/T ;       % remove the mean
sps=zeros(1,w/2) ;     % smoothed power spectrum
spsold=sps ;
nps=zeros(1,w/2) ;     % noise estimate power spectrum

sn=zeros(cnt+1,w/2) ;
for i=0:cnt,                  % cycle over windows
 ind1=1+i*s ; ind2=w+i*s ;    % start and end indexes
% disp(['Frame ',num2str(i)]) ;
 xw=x(ind1:ind2) ;            % cut out the window
 sp=fft(xw) ;
 sp=sp(1:w/2) ;               % cut out the symetric half
 sp=sp.*conj(sp) ;            % we have the power spectrum here
 spsold=sps ;
 sps=alpha*sps+(1-alpha)*sp ;           % calculate power spectrum
 nps=gamma*nps+(1-gamma)/(1-beta)*(sps-beta*spsold) ;
 nps=min(nps,sps) ;           % new noise estimate
 sn(i+1,:)=nps ;
end ; 
 

