function ft=havlena(y,u,c)
% Implements Mr.Havlena's simultaneous parameter and state 
% Kalman filter estimator

T=length(y) ;
n=length(c) ; % Assume the system to be of the order n


% Initial joint parameter and state vector fi=[b a x], b is n+1, 
% a is n and x is also n elements long
fi=zeros(1,3*n+1) ;
% We shall store it at every time point into the following array
ft=zeros(T,3*n+1) ;

% Covariance matrix
C=1e20*eye(3*n+1) ;

% Here the estimation begins
for i=1:T,
 % for each sample
 h=[u(i) zeros(1,2*n) 1 zeros(1,n-1)] ;
 % calculate Kalman gain
 k=h*C/(1+h*C*h') ;
 % calculate the output error
 oe=y(i)-h*fi' ;
 % perform the data step - i.e. improve the estimate
 fi=fi+k*oe ;
 C=C-(C*h'*h*C)/(1+h*C*h') ;
 % save the estimate
 ft(i,:)=fi ;
 % perform the time step
 fi(2*n+2:3*n)=fi(2*n+3:3*n+1) ; fi(3*n+1)=0 ;
 fi(2*n+2:3*n+1)=fi(2*n+2:3*n+1)-y(i)*fi(n+2:2*n+1)+u(i)*fi(2:n+1)...
  -oe*c ;
end ;
 
 
 