function [ y, e ]=kalmanrs(z,A,C,Q,r) 
% Performs bidirectional Kalman filtering for input-less white-noise driven 
% system with one output, system matrix A, output matrix C, process noise 
% covariance matrix Q and measurement noise covariance r.  
% z is the measured output and y is the estimated state (unlike kalmanr) 
% y(t,:) gives the state estimate at time t
% Usage: [y e]=kalmanrs(z,A,C,Q,r)

n=length(A) ;
T=length(z) ;

% initial uninformative guess
P=dlyap1(A,Q) ;
x=zeros(n,1) ;
xf=zeros(n,T) ;
Pf=zeros(T,n*n) ;
xp=zeros(n,T) ;
Pp=zeros(T,n*n) ;
y=zeros(T,n) ;
e=zeros(1,T) ;

% Forward run
for i=1:T,
 % Data step
 e(i)=z(i)-C*x ;          % prediction error
 L=P*C' / (C*P*C'+r) ; % Kalman gain
 x=x+L*e(i) ;             % correct the state estimate
 P=P-L*C*P ;

 Pf(i,:)=reshape(P,1,n*n) ;
 xf(:,i)=x ;              % filtered estimate
 
 % Time step
 x=A*x ;
 P=A*P*A'+Q ;

 xp(:,i)=x ;              % predicted estimate
 Pp(i,:)=reshape(P,1,n*n) ;
end ;

% Backward run
xl=x ;
for i=T:-1:1,
  Pfilt=reshape(Pf(i,:),n,n) ;
  Ppred=reshape(Pp(i,:),n,n) ;
  if rank(Ppred)<n,
     x=xf(:,i) ;
  else   
     F=(Pfilt*A') / Ppred ;
     x=xf(:,i)+F*(xl-xp(:,i)) ;
  end ;   
  xl=x ;
  y(i,:)=x' ;
end ;


