function [ y, e ]=kalmanrsi1(z,A,C,Q,r,P) 
% Performs bidirectional Kalman filtering for input-less white-noise driven 
% system with one output, system matrix A, output matrix C, process noise 
% covariance matrix Q, measurement noise covariance r and initial state
% estimate covariance P  
% z is the measured output and y is the estimated state (unlike kalmanr) 
% y(t,:) gives the state estimate at time t
% Usage: [y e]=kalmanrsi1(z,A,C,Q,r)

n=length(A) ;
T=length(z) ;

% initial uninformative guess
x=zeros(n,1) ;
xf=zeros(n,T) ;
Pf=zeros(n,n) ;
xp=zeros(n,T) ;
Pp=zeros(n,n) ;
Fp=zeros(T,n*n) ;
y=zeros(T,n) ;
e=zeros(1,T) ;

% Forward run
for i=1:T,
 % Data step
 e(i)=z(i)-C*x ;          % prediction error
 L=P*C' / (C*P*C'+r) ; % Kalman gain
 x=x+L*e(i) ;             % correct the state estimate
 P=P-L*C*P ;

 Pf=P ;
 xf(:,i)=x ;              % filtered estimate
 
 % Time step
 x=A*x ;
 P=A*P*A'+Q ;

 xp(:,i)=x ;              % predicted estimate
 F=(Pf*A')/P ;
 Fp(i,:)=reshape(F,1,n*n) ;
end ;

% Backward run
xl=x ;
for i=T:-1:1,
  F=reshape(Fp(i,:),n,n) ;
  x=xf(:,i)+F*(xl-xp(:,i)) ;
  xl=x ;
  y(i,:)=x' ;
end ;


