function [ y, e ]=kalmans(z,A,C,Q,r) 
% Performs unidirectional Kalman filtering for input-less white-noise driven 
% system with one output, system matrix A, output matrix C, process noise 
% covariance matrix Q and measurement noise covariance r.  
% z is the measured output and y is the estimated state (unlike kalmanf) 
% y(t,:) gives the state estimate at time t
% Usage: [y e]=kalmanrs(z,A,C,Q,r)

n=length(A) ;
T=length(z) ;

% initial uninformative guess
P=dlyap1(A,Q) ;
x=zeros(n,1) ;
y=zeros(T,n) ;
e=zeros(1,T) ;

% Forward run
for i=1:T,
 % Data step
 e(i)=z(i)-C*x ;          % prediction error
 L=P*C' / (C*P*C'+r) ; % Kalman gain
 x=x+L*e(i) ;             % correct the state estimate
 P=P-L*C*P ;

 % Time step
 x=A*x ;
 P=A*P*A'+Q ;

 y(i,:)=x' ;
end ;




