function [ys,e]=kalmk(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1] x discrete system. 
% k is the Kalman gain
% Usage: [ys,e]=kalmk(z,A,k)
% ys,e,z,k are row vectors

T=length(z) ;
n=length(A) ;
k=k' ;

x=z(1)*zeros(n,1) ; % initial state estimate mean

for i=1:T,  
  ys(i)=x(n) ;         % prediction of y 
  e(i)=z(i)-ys(i)   ;  % the inovation
  x=x+k*e(i)        ;  % make the data step
  ys(i)=x(n) ;         % estimation of y
  x=A*x ;              % make the time step  
  end ;

 
 





