function [ys,e]=kalmke(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1] x discrete system. 
% Iteratively estimates the Kalman gain k and calls kalmk
% k is the first estimate of the gain
% Usage: [ys,e]=kalmke(z,A,k)
% ys,e,z,k are row vectors

T=length(z) ;
n=length(A) ;
k=k';

h=[ zeros(1,n-1) 1] ; % output vector h 

[ys e]=kalmk(z,A,k') ;  % estimate the output
r=iswhite(e) 
k'
i=0 ;

while r<1,  % as long as the innovation is not white
  i=i+1 ;
  if i>10, 
    disp('kalmke: maximum (10) number of iteration reached.') ;
    break ;
    end ;
  cr=corrn(e,n+1)  ; % autocorrelation of the inovation sequence
  c=cr(2:n+1)/cr(1) ; 
  % construct the matrix q
  aikh=A*(eye(n)-k*h) ; 
  it=eye(n) ;
  q=zeros(n) ; 
  for j=1:n,
     q(j,:)=h*it*A ;
     it=it*aikh ;
     end ;
  % Compute new gain
  k=k+q \ c ;
  [ys e]=kalmk(z,A,k') ;
  r=iswhite(e) ,k'
  end ;
     




