function [ys,e,kout]=kalmke1(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1]*x discrete system. 
% Iteratively estimates the Kalman gain k and calls kalmk
% k is the first estimate of the gain. kout is the resulting gain
% Tries to use more autocorrelation coefficients than n, currently 20.
% Usage: [ys,e,kout]=kalmke1(z,A,k)
% ys,e,z,k,kout are row vectors
% 
% This function gives generally better results than kalmke. It does not use the
% autocorrelation function to determine the whiteness of the inovation
% sequence, instead it filters until convergence (of k) is attained.
% Usually, this is faster.

T=length(z) ;
n=length(A) ;
k=k';
m=max(50,n) ; % number of autocorr coefs used
prec=1e-2 ;   % the desired precision

h=[ zeros(1,n-1) 1] ; % output vector h 

[ys e]=kalmk(z,A,k') ;  % estimate the output
%plot(ys) ;

  cr=corrn(e,m+1)  ; % autocorrelation of the inovation sequence
  c=cr(2:m+1)/cr(1) ; 
  % construct the matrix q
  aikh=A*(eye(n)-k*h) ; 
  max(abs(roots(poly(aikh))))
  pause
  it=eye(n) ;
  q=zeros(n) ; 
  for j=1:m,
     q(j,:)=h*it*A ;
     it=it*aikh ;
     end ;
  k1=q \ c 
  
  mht=k*cr(1)+(q \ cr(2:m+1)) ;
  r=cr(1)-h*mht

  md=eye(n) ;
  
  while max(abs(k1-k))<1e-6,
  aikh=A*(eye(n)-k1*h) ;  
  mdo=2*md ;
  while max(max(abs(mdo-md)))<1e-6,
     mdo=md ;
     md=aikh*md*aikh'-A*(k1-k)*cr(1)*(A*(k1-k))' ;
  end ;
  md, pause 
  m2ht=mht+md*h' ;
  k2=m2ht*inv(h*m2ht+r) 
  mht=m2ht ; k=k1 ; k1=k2 ;
  end ;

kout=k' ;
[ys e]=kalmk(z,A,k') ;  % estimate the output




