function [ys,e,kout]=kalmkebsvd(z,A)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=A*x(t), ys=[0..0 1]*x discrete system. 
% First estimate measurement noise, than iteratively estimates 
% the process noise and calls kalmk
% Usage: [ys,e,kout]=kalmke(z,A,k)
% ys,e,z,k,kout are row vectors
% 

T=length(z) ;
n=length(A) ;

% Fix for unstable matrixes
while (max(abs(roots(poly(A))))>0.999), A=0.99*A ; end ;

m=max(50,n) ; % number of autocorr coefs used
prec=1e-2 ;   % the desired precision
r=1 ;

   opt=foptions ; opt(1)=0 ; opt(2)=prec ;
   i=fmin('kbfoptsvd',-20,log(r)/log(10),opt,A,r,z) ;  
   q=10^(i) ;
   P=eye(n) ;
   Pold=2*P ;
   while max(max(abs(P-Pold)))>1e-6,
      Pold=P ;	
%     P=A*(P-P*h'*inv(h*P*h'+r)*h*P)*A'+h*q*h' ;
      P=A*(P-P(:,n)/(P(n,n)+r)*P(n,:))*A' ;
      P(n,n)=P(n,n)+q ;
      end ;
%   P
   k=P(:,n)/(P(n,n)+r) ;
   [ys e]=kalmksvd(z,A,k') ;   
   kout=k' ;

