function [ys,e,kout]=kalmkee(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1]*x discrete system. 
% Iteratively estimates the Kalman gain k and calls kalmk
% k is the first estimate of the gain. kout is the resulting gain
% Tries to use more autocorrelation coefficients than n, currently 20.
% Usage: [ys,e,kout]=kalmkee(z,A,k)
% ys,e,z,k,kout are row vectors
% 
% This function performs generally better than kalmke. It does not use the
% autocorrelation function to determine the whiteness of the inovation
% sequence, instead it filters until convergence (of k) is attained.
% Usually, this is faster.

T=length(z) ;
n=length(A) ;
k=k';
m=max(200,n) ; % number of autocorr coefs used
prec=1e-2 ;   % the desired precision

% Fix for unstable matrixes
while (max(abs(roots(poly(A))))>0.99), A=0.99*A ; end ;


h=[ zeros(1,n-1) 1] ; % output vector h 

[ys e]=kalmk(z,A,k') ;  % estimate the output
%r=iswhite(e)           % here we could find, if e is white
%k'
i=0 ;
kold=10*k+1e6*ones(n,1) ;

while sum(abs(kold-k))/sum(abs(k)+1e-6)>prec,  
  % as long as k changes substantially
  % alternatively 
%while r<1,
  i=i+1 ;
%  plot(ys) ; pause ;
  disp(['Iteration ',num2str(i)]) ;
  kold=k ;
  if i>100, 
    disp('kalmkee: maximum (100) number of iteration reached.') ;
    break ;
    end ;
  cr=corrs(e,m+1)  ; % autocorrelation of the inovation sequence
%  cr=corrs(e,m+7)  ; % autocorrelation of the inovation sequence
%  cr=corrs(cr',m+6) ;
%  cr=corrs(cr',m+5) ;
%  cr=corrs(cr',m+4) ;
%  cr=corrs(cr',m+3) ;
%  cr=corrs(cr',m+2) ;
%  cr=corrs(cr',m+1) ;

  c=cr(2:m+1)/cr(1) ; 
  % construct the matrix q
  aikh=A*(eye(n)-k*h) ; 
  it=eye(n) ;
  q=zeros(n) ; 
  for j=1:m,
     q(j,:)=h*it*A ;
     it=it*aikh ;
     end ;
  % Compute new gain
  k=k+q \ c ;   % \ represents LSM solution


%  maxr=max(abs(roots(poly(A*(eye(n)-k*h))))) ;
%  while maxr>=1,
%      k=0.1*abs(k) ;
%      maxr=max(abs(roots(poly(A*(eye(n)-k*h))))) ;
%      end ;
   % keep k in bounds	
   for ki=1:n, 
     if isnan(k(ki)), 
       disp('Divergence in k detected, aborting') ;
       k=zeros(n,1) ;
       break ;
       end ;
   end ;


  [ys e]=kalmk(z,A,k') ;
%  r=iswhite(e) 
   k'
  end ;

kout=k' ;     
















