function [ys,e]=kalmkee1(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1] x discrete system. 
% Iteratively estimates the Kalman gain k and calls kalmk
% k is the first estimate
% Tries to use more autocorrelation coefficients.
% Usage: [ys,e]=kalmkee1(z,A,k)
% ys,e,z,k are row vectors

T=length(z) ;
n=length(A) ;
k=k';
m=min(20,n) ; % number of autocorr coefs used

h=[ zeros(1,n-1) 1] ; % output vector h 

[ys e]=kalmk(z,A,k') ;  % estimate the output
%r=iswhite(e) 
k'
i=0 ;
kold=10*k ;

while sum(abs(kold-k))/sum(abs(k))>1e-6,  
  % as long as k changes substantially
  plot(ys) ;
  pause ;
  i=i+1 ;
  kold=k ;
  if i>100, 
    disp('kalmkee1: maximum (100) number of iteration reached.') ;
    break ;
    end ;
  cr=corrn(e,m+1)  ; % autocorrelation of the inovation sequence
  c=cr(2:m+1)/cr(1) ; 
  % construct the matrix q
  aikh=A*(eye(n)-k*h) ; 
  it=eye(n) ;
  q=zeros(n) ; 
  for j=1:m,
     q(j,:)=h*it*A ;
     it=it*aikh ;
     end ;
  % Compute new gain
  k=k+(q \ c) ;   % \ represents LSM solution
  [ys e]=kalmk(z,A,k') ;
%  r=iswhite(e) ,
   max(ys),k'
  end ;
     




