function [ys,e]=kalmkg(z,a,kg)
% Performs steady-state Kalman filtering with gradient gain estimation
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t)=sum_i a_i x(t-i), ys=[0..0 1] x discrete system. 
% k is the Kalman gain factor
% Usage: [ys,e]=kalmk(z,A,k)
% ys,e,z,k are row vectors

T=length(z) ;
n=length(a) ;
A=ar2tm(a) ;
k=a'*kg  ;

x=z(1)*ones(n,1) ; % initial state estimate mean
g=ones(1,T) ;
lasti=0 ; % last innovation

for i=1:T,  
  ys(i)=x(n) ;         % prediction of y 
  e(i)=z(i)-ys(i)   ;  % the inovation
  g(i)=e(i)*lasti
  x=x+k*e(i)        ;  % make the data step
  ys(i)=x(n) ;         % estimation of y
  x=A*x ;              % make the time step  
  end ;

 
 





