function [ys,e,kout]=kalmkre(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1]*x discrete system. 
% Iteratively estimates the Kalman gain k and calls kalmk
% k is the first estimate of the gain. kout is the resulting gain
% Tries to use more autocorrelation coefficients than n, currently 2-0.
% Weights to improve robustness
% Usage: [ys,e,kout]=kalmkre(z,A,k)
% ys,e,z,k,kout are row vectors
% 
% This function gives generally better than kalmke. It does not use the
% autocorrelation function to determine the whiteness of the inovation
% sequence, instead it filters until convergence (of k) is attained.
% Usually, this is faster.

T=length(z) ;
n=length(A) ;
k=k';
m=max(200,n) ; % number of autocorr coefs used
prec=1e-2 ;   % the desired precision

h=[ zeros(1,n-1) 1] ; % output vector h 

[ys e]=kalmk(z,A,k') ;  % estimate the output
%r=iswhite(e)           % here we could find, if e is white
i=0 ;
kold=10*k ;



while sum(abs(kold-k))/sum(abs(k))>prec,  
  % as long as k changes substantially
  % alternatively 
%while r<1,
  i=i+1 ;
  disp(['Iteration ',num2str(i)]) ;
  kold=k ;
  if i>20, 
    disp('kalmkee: maximum (20) number of iteration reached.') ;
    break ;
    end ;
  cr=corrn(e,m+1)  ; % autocorrelation of the inovation sequence
  c=cr(2:m+1)/cr(1) ; 

  % construct the matrix q
  aikh=A*(eye(n)-k*h) ; 
  it=eye(n) ;
  q=zeros(m,n) ; 
  for j=1:m,
     q(j,:)=h*it*A * (1/j);
     c(j)=c(j)/j ;
     it=it*aikh ;
     end ;

  % Compute new gain
  
  k=k+q \ c ;   % \ represents LSM solution
  [ys e]=kalmk(z,A,k') ;
   k'
  end ;

kout=k' ;     




