function [ys,e]=kalmksvd(z,A,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1] x discrete system. 
% k is the Kalman gain
% Usage: [ys,e]=kalmk(z,A,k)
% ys,e,z,k are row vectors

T=length(z) ;
n=length(A) ;
k=k' ;

M=zeros(T,n) ; % M*x0-p = err
p=zeros(T,1) ;   % the right side of the equation
yhat=zeros(T) ;
w=eye(n) ;    % the actual estimate
wa=zeros(n,1) ;         % the absolute factor     
IK=zeros(n) ; IK(:,n)=k ; IK=A*(eye(n)-IK) ;
for i=1:T,
  M(i,:)=-w(n,:) ; p(i)=wa(n) ; 
  % Compute y(k|k-1) -> y(k+1|k)
  wa=A*(wa+k*(z(i)-wa(n))) ;
  w=IK*w ;  
  end ;
p=z'-p ;

% x0=M \ p ;
% To get rid of the warnings

[U,S,V]=svd(M,0) ;

ms=max(diag(S))*1e-12 ;
for i=1:n, 
  if S(i,i)>ms, S(i,i)=1/S(i,i) ; 
           else S(i,i)=0 ;
  end ;
end ;

x0=V*S*U'*p ;

e=p-M*x0 ; ys=z-e' ;


 
 





