function [ys,e,g,kout]=kalmkw(z,a,k)
% Performs steady-state Kalman filtering
% assumes z=ys+e, where e is the innovation and ys is generated
% by the x(t+1)=Ax(t), ys=[0..0 1] x discrete system. 
% k is the Kalman gain
% Usage: [ys,e]=kalmkw(z,A,k)
% ys,e,z,k are row vectors
%
% Kalman gain improvements by gradient algorithm
gamma=1e-9 ;

T=length(z) ;
n=length(a) ;
A=ar2tm(a) ;
k=k' ;

if n~=4, 
        error('So far, kalmkw works only for n=4') ;
end ;

x=z(1)*ones(n,1) ; % initial state estimate mean
x=zeros(n,1) ;
g=zeros(T,n) ;
se=0 ;

for i=1:T,  
  ys(i)=x(n) ;         % prediction of y 
  e(i)=z(i)-ys(i)   ;  % the inovation
  x=x+k*e(i)        ;  % make the data step
  se=se+e(i)*e(i) ;

% the gradients
  if i>n,
    g(i,4)=-2*e(i)*e(i-1) ;
    g(i,3)=-2*e(i)*(a(2)*e(i-2)+a(3)*e(i-3)+a(4)*e(i-4)) ;
    g(i,2)=-2*e(i)*(a(3)*e(i-2)+a(4)*e(i-3)) ;
    g(i,1)=-2*e(i)*a(4)*e(i-2) ;
% change the Kalman gain
    k=k+gamma*g(i,:)' ;    
  end ;

  ys(i)=x(n) ;         % estimation of y
  x=A*x ;              % make the time step  
  end ;
kout=k' ;
 
figure(2) ; plot(ys') ;   
sum(g) 
se



