function sn=procsig2(x)
% This function estimates the power spectrum of the speech signal
% It is based on getpns2
% Usage: sn=procsig2(x)

% Set some constants first
w=256   ; % window length, should be power of two
ovrf=4  ; % overlap factor

omin=1.5 ; % estimate bias factor

lnga=20 ; 
lngb=10 ;
lng=lnga*lngb ; % length of the window for minimum evaluation
alpha=0.5 ;     % smoothing factor 

s=w/ovrf ; % window shift

x=x(:)' ;
T=length(x) ;

cnt=floor((T-w)/s) ; % how many windows
h=hanning1(w) ;       % window weights
h=w/ovrf/sum(h)*h ;  % adjust to proper scaling


x=x-sum(x)/T ;       % remove the mean
sps=zeros(1,w/2) ;   % smoothed power spectrum
cb=zeros(lngb,w/2) ; % circular buffer of sps history
cbi=1 ;              % index to circular buffer
cbmin=zeros(1,w/2) ; % current minimum of the circular buffer
lsmin=zeros(1,w/2) ; % current minimum of the last segment
lsc=lnga ;           % length of the last segment
nps=zeros(1,w/2) ;   % noise estimate power spectrum

sn=zeros(cnt+1,w/2) ;
for i=0:cnt,                  % cycle over windows
 ind1=1+i*s ; ind2=w+i*s ;    % start and end indexes
% disp(['Frame ',num2str(i)]) ;
 xw=x(ind1:ind2) ;            % cut out the window
 sp=fft(xw) ;
 sp=sp(1:w/2) ;               % cut out the symetric half
 sp=sp.*conj(sp) ;            % we have the power spectrum here
 sps=alpha*sps+(1-alpha)*sp ; % calculate power spectrum
 if lsc>=lnga,                % last window of the segment
   cb(cbi,:)=lsmin ;          % circular buffer update
   cbi=mod(cbi,lngb)+1 ;      % pointer update
   cbmin=min(cb) ;
   lsmin=sps ;
   lsc=1 ;
 else  
   lsmin=min(lsmin,sps) ;
   lsc=lsc+1 ;
 end ;
 
 nps=omin*min(lsmin,cbmin) ;       % new noise estimate

 % smoothed speech power spectrum estimate
 sss=max([zeros(1,w/2) ; sps-nps]) ; 
 sn(i+1,:)=sss ;
 
end ; 

