function yf=procsig4(x)
% This function performs the total processing of the input vector
% using spectral subtraction filtering using Martin's algorithm
% It is based on procsig2 and getpns2
% Usage: yf=procsig4(x)

% Set some constants first
w=256   ; % window length, should be power of two
ovrf=4  ; % overlap factor

omin=1.5 ; % estimate bias factor

lnga=8 ; 
lngb=7 ;
lng=lnga*lngb ; % length of the window for minimum evaluation
alpha=0.9 ;     % smoothing factor 

s=w/ovrf ; % window shift

x=x(:)' ;
T=length(x) ;

cnt=floor((T-w)/s) ; % how many windows
h=hanning1(w) ;       % window weights
h=w/ovrf/sum(h)*h ;  % adjust to proper scaling


x=x-sum(x)/T ;       % remove the mean
sps=zeros(1,w/2) ;   % smoothed power spectrum
cb=zeros(lngb,w/2) ; % circular buffer of sps history
cbi=1 ;              % index to circular buffer
cbmin=zeros(1,w/2) ; % current minimum of the circular buffer
lsmin=zeros(1,w/2) ; % current minimum of the last segment
lsc=lnga ;           % length of the last segment
nps=zeros(1,w/2) ;   % noise estimate power spectrum
yf=zeros(1,T) ;      % output

sn=zeros(cnt+1,w/2) ;
for i=0:cnt,                  % cycle over windows
 ind1=1+i*s ; ind2=w+i*s ;    % start and end indexes
 disp(['Frame ',num2str(i)]) ;
 xw=x(ind1:ind2) ;            % cut out the window
 sp=fft(xw) ;
 spangle=angle(sp) ;
 sp=sp(1:w/2) ;               % cut out the symetric half
 sp=sp.*conj(sp) ;            % we have the power spectrum here
 sps=alpha*sps+(1-alpha)*sp ; % calculate power spectrum
 if lsc>=lnga,                % last window of the segment
   cb(cbi,:)=lsmin ;          % circular buffer update
   cbi=mod(cbi,lngb)+1 ;      % pointer update
   cbmin=min(cb) ;
   lsmin=sps ;
   lsc=1 ;
 else  
   lsmin=min(lsmin,sps) ;
   lsc=lsc+1 ;
 end ;
 
 nps=omin*min(lsmin,cbmin) ;       % new noise estimate

 % smoothed speech power spectrum estimate
 sss=max([zeros(1,w/2) ; sp-nps]) ; 

 % now we have the speech power spectrum in sss 
 % and the noise power spectrum in nps

if i>lng, 
 s1=(sp-nps)./nps ;
 s1=max([ones(1,w/2) ; s1]) ;
 s1=min([100*ones(1,w/2) ; s1]) ;
 s1=4-1.5*log10(s1) ;
 s1=ones(1,w/2)-sqrt(s1 .* nps./sps) ;
 s1=max([zeros(1,w/2) ; s1]) ;
 sss=sp.*s1 ;
end ; 
  
 sss=sqrt([sss sss(w/2:-1:1)]).*exp(j*spangle) ;
 y=real(ifft(sss)) ;
 yf(ind1:ind2)=yf(ind1:ind2)+h'.*y ;
end ; 

