function yf=procsig5(x)
% This function performs the total processing of the input vector
% using Kalman filtering using Martin's algorithm
% It is based on procsig2 and getpns2
% We currently use darkalmc optimal bidirectional Kalman filtering
% and tstlswc TLS parameter estimation
% Usage: yf=procsig5(x)

% Set some constants first
w=256   ; % window length, should be power of two
ovrf=4  ; % overlap factor

omin=1.5 ; % estimate bias factor

ns=16 ; nn=16 ;

lnga=10 ; 
lngb=10 ;
lng=lnga*lngb ; % length of the window for minimum evaluation
alpha=0.9 ;     % smoothing factor 
% alpha=0.7 ;     % smoothing factor 
r=1e-6 ;        % supposed measurement error covariance 

s=w/ovrf ; % window shift

x=x(:)' ;
T=length(x) ;

cnt=floor((T-w)/s) ; % how many windows
h=hanning1(w) ;       % window weights
h=w/ovrf/sum(h)*h ;  % adjust to proper scaling


meanx=sum(x)/T ;
x=x-meanx ;          % remove the mean
maxx=max(abs(x)) ;   % calculate the maximum value
x=x/maxx ;           % scale accordingly

sps=zeros(1,w/2) ;   % smoothed power spectrum
cb=zeros(lngb,w/2) ; % circular buffer of sps history
cbi=1 ;              % index to circular buffer
cbmin=zeros(1,w/2) ; % current minimum of the circular buffer
lsmin=zeros(1,w/2) ; % current minimum of the last segment
lsc=lnga ;           % length of the last segment
nps=zeros(1,w/2) ;   % noise estimate power spectrum
yf=zeros(1,T) ;      % output

sn=zeros(cnt+1,w/2) ;
for i=0:cnt,                  % cycle over windows
 ind1=1+i*s ; ind2=w+i*s ;    % start and end indexes
 disp(['Frame ',num2str(i)]) ;
 xw=x(ind1:ind2) ;            % cut out the window
 sp=fft(xw) ;
 spangle=angle(sp) ;
 sp=sp(1:w/2) ;               % cut out the symetric half
 sp=sp.*conj(sp) ;            % we have the power spectrum here
 sps=alpha*sps+(1-alpha)*sp ; % calculate power spectrum
 if lsc>=lnga,                % last window of the segment
   cb(cbi,:)=lsmin ;          % circular buffer update
   cbi=mod(cbi,lngb)+1 ;      % pointer update
   cbmin=min(cb) ;
   lsmin=sps ;
   lsc=1 ;
 else  
   lsmin=min(lsmin,sps) ;
   lsc=lsc+1 ;
 end ;
 
 nps=omin*min(lsmin,cbmin) ;       % new noise estimate

 % smoothed speech power spectrum estimate
% sss=max([zeros(1,w/2) ; sp-nps]) ; 
% sss=sp-nps ;
sss=sps-nps ;

 % now we have the speech power spectrum in sss 
 % and the noise power spectrum in nps

if i>lng, 
% s1=(sp-nps)./nps ;
% s1=max([ones(1,w/2) ; s1]) ;
% s1=min([100*ones(1,w/2) ; s1]) ;
% s1=4-1.5*log10(s1) ;
% s1=ones(1,w/2)-sqrt(s1 .* nps./sps) ;
% s1=max([zeros(1,w/2) ; s1]) ;
% sss=sp.*s1 ;
 
 % the corrected noise power spectrum is now in sss ;
% [as bs cs]=s2aex(sss,ns) ; % get speech parameters
% [an bn cn]=s2aex(nps,nn) ; % get noise parameters
 [as bs cs]=s2ae(sss,ns) ; % get speech parameters
 [an bn cn]=s2ae(nps,nn) ; % get noise parameters

 
y=darkalm3c(xw,as,bs,cs,an,bn,cn,r) ; 
% y=darkalmc(xw,as,bs,cs,an,bn,cn,r) ; 
%y=darkalmbc(xw,as,bs,cs,an,bn,cn,r) ; 
  
xmax=max(abs(xw)) ; ymax=max(abs(y)) ;

 if ymax>1.0*xmax,
   disp('Falling back to darkalmsc') ;
   y=darkalmsc(xw,as,bs,cs,an,bn,cn) ; 
 end ;
 yf(ind1:ind2)=yf(ind1:ind2)+h'.*y ;
end ;
  
end ; 
yf=yf*maxx+meanx ;
