function yf=procsig7(x)
% This function performs the total processing of the input vector
% using Kalman filtering using Martin's algorithm
% It is based on procsig2 and getpns2
% We currently use fast darkalm3c optimal bidirectional Kalman filtering
% and darkalms as a fall back
% 
% We use maximum entropy frequency estimation
%
% Does not work well
% Usage: yf=procsig7(x)

% Set some constants first
w=256   ; % window length, should be power of two
ovrf=4  ; % overlap factor

omin=1.5 ; % estimate bias factor

ns=8 ; nn=8 ;

lnga=8 ; 
lngb=8 ;
lng=lnga*lngb ; % length of the window for minimum evaluation
alpha=0.9 ;     % smoothing factor for noise spectrum estimation
alphas=0 ;   % smoothing factor for immediate spectrum
beta=0.9 ;      % additional smoothing of noise spectra
r=1e-6 ;        % supposed measurement error covariance 

s=w/ovrf ; % window shift

x=x(:)' ;
T=length(x) ;

cnt=floor((T-w)/s) ; % how many windows
h=hanning1(w) ;       % window weights
h=w/ovrf/sum(h)*h ;  % adjust to proper scaling


meanx=sum(x)/T ;
x=x-meanx ;          % remove the mean
maxx=max(abs(x)) ;   % calculate the maximum value
x=x/maxx ;           % scale accordingly

sps=zeros(1,w/2+1) ;   % smoothed power spectrum
ssps=zeros(1,w/2+1) ;   % smoothed speech power spectrum
cb=zeros(lngb,w/2+1) ; % circular buffer of sps history
cbi=1 ;              % index to circular buffer
cbmin=zeros(1,w/2+1) ; % current minimum of the circular buffer
lsmin=zeros(1,w/2+1) ; % current minimum of the last segment
lsc=lnga ;           % length of the last segment
nps=zeros(1,w/2+1) ;   % noise estimate power spectrum
snps=zeros(1,w/2+1) ;  % smoothed noise estimate power spectrum
yf=zeros(1,T) ;      % output

sn=zeros(cnt+1,w/2+1) ;
for i=0:cnt,                  % cycle over windows
 ind1=1+i*s ; ind2=w+i*s ;    % start and end indexes
 disp(['Frame ',num2str(i)]) ;
 xw=x(ind1:ind2) ;            % cut out the window
% sp=fft(xw.*h'*ovrf) ;
 sp=memsp(xw,16,T)' ;
 sp=sp(1:w/2+1) ;               % cut out the symetric half
 sp=sp.*conj(sp) ;            % we have the power spectrum here
 sps=alpha*sps+(1-alpha)*sp ; % calculate power spectrum
 ssps=alphas*ssps+(1-alphas)*sp ; % calculate power spectrum
 if lsc>=lnga,                % last window of the segment
   cb(cbi,:)=lsmin ;          % circular buffer update
   cbi=mod(cbi,lngb)+1 ;      % pointer update
   cbmin=min(cb) ;
   lsmin=sps ;
   lsc=1 ;
 else  
   lsmin=min(lsmin,sps) ;
   lsc=lsc+1 ;
 end ;
 
 nps=omin*min(lsmin,cbmin) ;       % new noise estimate
 snps=beta*snps+(1-beta)*nps ;     % calculate power spectrum
 
 
 % now we have the speech power spectrum in sss 
 % and the noise power spectrum in snps

if i>=lng, 

if 1,
  % Martinuv algoritmus  
  s1=(ssps-snps)./snps ;
  s1=max([ones(1,w/2+1) ; s1]) ;
  s1=min([100*ones(1,w/2+1) ; s1]) ;
  s1=4-1.5*log10(s1) ;
  % s1=2-1.0*log10(s1) ;
  s1=ones(1,w/2+1)-sqrt(s1 .* snps./ssps) ;
  % s1=ones(1,w/2+1)-sqrt(snps./sp) ;
  s1=max([zeros(1,w/2+1) ; s1]).^2 ;
  sss=s1.*ssps ;
else
  if 1,  
    % Wiener  
    s1=ssps-snps ;
    s1=max([zeros(1,w/2+1) ; s1]) ;
    sss=(s1.^2)./ssps ; 
  else
    % HWR
    sss=ssps-snps ;
    sss=max([zeros(1,w/2+1) ; sss]) ;
  end ; 
end ;

% the corrected noise power spectrum is now in sss ;
% [as bs cs]=s2aex(sss,ns) ; % get speech parameters
% [an bn cn]=s2aex(snps,nn) ; % get noise parameters
 [as bs cs]=s2ae([sss sss(w/2:-1:2)],ns) ;  % get speech parameters
 [an bn cn]=s2ae([snps snps(w/2:-1:2)],nn) ; % get noise parameters
% disp([ num2str(bs) ' ' num2str(bn) ' ' num2str(sss(1)) ' ' ...
%      num2str(snps(1))]) ;
 
y=darkalm3c(xw,as,bs,cs,an,bn,cn,r) ; 
% y=darkalmc(xw,as,bs,cs,an,bn,cn,r) ; 
%y=darkalmbc(xw,as,bs,cs,an,bn,cn,r) ; 
  
xmax=max(abs(xw)) ; ymax=max(abs(y)) ;

 if ymax>1.0*xmax,
   y=darkalmsc(xw,as,bs,cs,an,bn,cn) ;
   nymax=max(abs(y)) ;
   disp(['Falling back to darkalmsc xmax=' num2str(xmax) ' ymax=' ...
	 num2str(ymax) ' nymax=' num2str(nymax)]) ;
 end ;
 yf(ind1:ind2)=yf(ind1:ind2)+h'.*y ;
end ;
  
end ; 
yf=yf*maxx+meanx ;
