function savewav(filename,sig,fsamp,bits,channels);

%Function SAVEWAV: saving vector as WAV-formated signal file
%
%Usage: savewav(filename,sig,fsamp,bits,channels);
%  filename  - desired name of WAV-file with signal 
%  sig       - collumn vector with signal 
%  fsamp     - sampling frequency of signal
%  bits      - number of bits per sample
%  channels  - number of channels (MONO signals - 1, STEREO signals - 2)
%       In the case of channels == 2, left and right channel data is written
%       in the vector in alternating order

% typedef unsigned int  word ;
% typedef unsigned long dword ;
%
% /*********************  Hlavicka WAV souboru   ***************************/

%   typedef struct {
%   char  acRIFF[4];       /* "RIFF"                                    */
%   dword dwRiffLen;       /* remaining bytes in File                   */
%   char  acWave[4];       /* "WAVE"                                    */
%   char  acFmt[4];        /* "fmt"                                     */
%   dword dwFmtLen;        /* 16, lenght of format block following      */
%   word  wFormatTag;      /* 1 (WAVE_FORMAT_PCM), others reserved      */
%   word  wChannels;       /* 1=mono, 2=stereo                          */
%   dword dwSamplesPerSecond; /* in Hz (11025, 22050,44100,..)          */
%   dword dwAvgBytesPerSec;/* (dwSamplesPerSecond * wBlockAlign)        */
%   word  wBlockAlign;     /* (wBitsPerSample * wChannels / 8)          */
%   word  wBitsPerSample;  /* 8 or 16                                   */
%   char  acData[4];       /* "DATA"                                    */
%   dword dwDataLen;       /* lenght of sample data (bytes!)            */
% } WAVEFILE ;

% /* In case of wChannels == 2, left and right channel data is found in alternating
% 	 sequence. */

%  File opening
%%%%%%%%%%%%%%%

F=fopen(filename,'w');
if F==-1, disp('Error: Signal file was not open !');  return; end; 


%  Writing of head of WAV-file with signal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

len=length(sig);

count=        fwrite(F,'RIFF','char');  
count=count + fwrite(F,36+len*bits/8,'ulong');
count=count + fwrite(F,'WAVEfmt ','char');
count=count + fwrite(F,16,'ulong');
count=count + fwrite(F,1,'ushort');
count=count + fwrite(F,channels,'ushort');
count=count + fwrite(F,fsamp,'ulong');
count=count + fwrite(F,fsamp*bits*channels/8,'ulong');
count=count + fwrite(F,bits*channels/8,'ushort');
count=count + fwrite(F,bits,'ushort');
count=count + fwrite(F,'data','char');
count=count + fwrite(F,len*bits/8,'ulong'); 

if ~(count==25), 
  disp('Error: Head was not successfully written !');
  return;
end;

%  Writing of signal samples to WAV-file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

maxval=max(abs(sig));
if maxval>32767,
  disp('Warning: Data exceeded range <-32767;32767>! Output may be distorted!');
end;

sig=floor(sig);

count=fwrite(F,sig,'short');  

if ~(count==len), 
  disp('Error: Head was not successfully written !');
  return;
end;


%  File closing end exit
%%%%%%%%%%%%%%%%%%%%%%%%

fclose(F);

disp(sprintf('%.0f were successfully written into %s file.',len,filename));
