function [a,y]=tstls6(x,n) 
% Toeplitz Structured Total Least Squares Algorithm
% 'x' containts measurements corrupted by white noise
% on output 'a' contains the coefficients of an AR system of the order 'n'
% generating output 'y', while (x-y) is minimised in Euclidean sense
% y(i)=a(1)*y(i-1)+a(2)*y(i-2)+...+a(n)*y(i-n)
% a=arg min sum((y(i)-x(i))^2)
%
% Based on STLNB algorithm from S. Van Huffel, Haesun Park, J. Ben Rosen: 
% 'Formulation and Solution of Structured Total Least Norm Problems for
% Parameter Estimation', IEEE Transactions on Signal Processing, 1996
%
% Iterative solution: linearizes the constraints at each point, computes
% the range of linearized constraint matrix and solves the corresponding
% reduced quadratic programming problem. Its solution serves as a basis for
% the next step.
%
% Not functioning very well
%
% Jan Kybic, 1997
%
% Usage: [a y]=tstls6(x,n)


x=x(:) ;      % make x a column vector
T=length(x) ; % number of available samples
m=T-n ;       % number of equations

prec=1e-6 ;   % desired precision
maxiter=50 ; % maximum number of iterations
 

% initialize the iterative procedure
a=burg1(x,n) ;
z=[a' ; x] ;
ex=[zeros(n,1) ; x] ;
J=crit(z,T,x,n) 

% meet the constraints first by changing just y's
[G u]=reduce(z,T,x,n) ;
G=G(:,n+1:2*n) ;
A=G'*G ;
b=G'*(ex-u-z) ;
dl=A\b ;
dz=G*dl+u ;
z=z+dz ;

for i=1:maxiter,
  % z is the current point we want to improve
  J0=crit(z,T,x,n) 
  [G u]=reduce(z,T,x,n) ;
  A=G'*G ;
  b=G'*(ex-u-z) ;
  dl=A\b ;
  dz=G*dl+u ;
  z1=z+dz ;
  [G u]=reduce(z1,T,x,n) ;
  G=G(:,n+1:2*n) ;
  A=G'*G ;
  b=G'*(ex-u-z) ;
  dl=A\b ;
  dz2=G*dl+u ;
  z2=z1+dz2 ;
  J1=crit(z2,T,x,n)
  small=1 ;
  while J1>J0,
    small=0.5*small ;
    z1=z+small*dz ;
    [G u]=reduce(z1,T,x,n) ;
    G=G(:,n+1:2*n) ;
    A=G'*G ;
    b=G'*(ex-u-z) ;
    dl=A\b ;
    dz2=G*dl+u ;
    z2=z1+dz2 ;
    J1=crit(z2,T,x,n)
    end ;
  z=z2 ;  
end ;

y=z(1:T) ;
a=z(T+1:T+n)' ;


% -------------------------------------------------
function J=crit(z,T,x,n)
% calculate the criteria
J=sum((z(n+1:T+n)-x).^2) ;

% -------------------------------------------------
function [G,u]=reduce(z,T,x,n)
% Given the column vector z=[a y], calculate G,u such as dz=G*dz(1:2*n)+u
% represents the linearized constraints
G=[eye(2*n) ; zeros(T-n,2*n)] ;
u=zeros(T+n,1) ;
y=z(n+1:T+n) ;
a=z(1:n) ;
for t=n+1:T,
  u(t+n)=[a(n:-1:1)' -1]*y(t-n:t) ;
  G(t+n,1:n)=y(t-1:-1:t-n)' ;
  G(t+n,n+1:2*n)=a(n:-1:1)'*G(t:t-1+n,n+1:2*n) ;
  u(t+n)=u(t+n)+a(n:-1:1)'*u(t:t-1+n) ;
end ;

% -------------------------------------------------
function [W,q]=calcWq(z,T,x,n,G,u,omega)
% calculates W=G'*A*G, q=A*u-x, given z=G*dz+u so that
% minimizing 0.5*dz'*W*dz+q'*dz would lead to the same solution as 
% minimizing 0.5*z'*A*z-2*x'*z
% while A represents 