function [a]=ttls(x,n) 
% Toeplitz Structured Total Least Squares Algorithm
% 'x' containts measurements corrupted by white noise
% on output 'a' contains the coefficients of an AR system of the order 'n'
% generating output 'y', while (x-y) is minimised in Euclidean sense
% y(i)=a(1)*y(i-1)+a(2)*y(i-2)+...+a(n)*y(i-n)
% a=arg min sum((y(i)-x(i))^2)
%
% Suboptimal algorithm neglecting the noise term correlation in Toeplitz matrix
% Does not work very well.
%
% Jan Kybic, 1997
%
% Usage: [a y]=ttls(x,n)


x=x(:) ;      % make x a column vector
T=length(x) ; % number of available samples
m=T-n ;       % number of equations

% construct the left hand side matrix
A=zeros(m,n) ;
for i=1:n, A(:,i)=x(i:i+m-1) ; end ;

b=x(n+1:T) ; % right hand side

% reduce rank
C=[A b] ;
[U S V]=svd(C,0) ;
% S(n+1,n+1)=0 ;
C=U*S*V' ;
A=C(:,1:n) ;
b=C(:,n+1) ;
a=A\b ;

a=a(n:-1:1)' ; % get the usual order